<?php
/**
 * 2012-2018 PrestaShop
 * NOTICE OF LICENSE
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    NetReviews SAS <contact@avis-verifies.com>
 * @copyright 2012-2022 NetReviews SAS
 * @license   NetReviews
 *
 * @version   Release: $Revision: 7.10.2
 *
 * @date      07/09/2022
 * International Registered Trademark & Property of NetReviews SAS
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'netreviews/NetReviewsModel.php';

class NetReviews extends Module
{
    public $html = null;
    public $iso_lang = null;
    public $id_lang = null;
    public $group_name = null;
    public $stats_product;
    public $consent = null;

    public function __construct()
    {
        $this->name = 'netreviews';
        $this->tab = 'advertising_marketing';

        $this->version = '7.10.2';
        $this->author = 'NetReviews';
        $this->need_instance = 0;
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('Verified Reviews');
        $this->description = $this->l('Collect, manage and publish reviews on your products and establishments but also on your brand in a few clicks thanks to Verified Reviews. 
            Bet on customer reviews to increase your e-reputation and increase your income.');
        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            $this->secure_key = Tools::encrypt($this->name);
        } else {
            $this->secure_key = Tools::hash($this->name);
        }
        $this->module_key = 'd63d28acbac0a249ec17b6394ac5a841';
        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            $isInstalled = self::isInstalled($this->name);
        } else {
            $isInstalled = self::isEnabled($this->name);
        }
        if ($isInstalled) {
            $this->id_lang = (int) Configuration::get('PS_LANG_DEFAULT');
            $this->iso_lang = pSQL(Language::getIsoById($this->id_lang));
        }
        $this->confirmUninstall = sprintf(
            $this->l('Are you sure you want to uninstall %s module?'),
            $this->displayName
        );
        $this->ps_versions_compliancy = ['min' => '1.5', 'max' => '1.7.99.99'];

        if (isset($this->context) && _PS_VERSION_ > '1.5') {
            $this->current_shop_id = $this->context->shop->getContextShopID();
            if (version_compare(_PS_VERSION_, '1.6', '>=') && version_compare(_PS_VERSION_, '1.7', '<') &&
                !Shop::isFeatureActive()) {
                $this->current_shop_id = null;
            }
        } elseif (version_compare(_PS_VERSION_, '1.5', '<')) { // _PS_VERSION_ < 1.4
            $this->current_shop_id = '';
        }
        if ('checked' == Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id)) {
            $this->id_lang = $this->context->language->id;
            $this->iso_lang = pSQL(Language::getIsoById($this->id_lang));
            $this->group_name = $this->getIdConfigurationGroup($this->iso_lang);
        }
        $this->all_multishops_reviews = false;
        $this->all_languages_reviews = false;

        // Retrocompatibility
        // if (version_compare(_PS_VERSION_, '1.5', '<') && version_compare(_PS_VERSION_, '1.4', '>=')) {
        //      $this->initContext();
        // }
    }

    public function install($keep = true)
    {
        if ($keep) {
            if (!($query = $this->createTables())) {
                $this->context->controller->errors[] = sprintf($this->l('SQL ERROR : %s | Query 
                    can\'t be executed. Maybe, check SQL user permissions.'), $query);
            }

            Configuration::updateValue('AV_IDWEBSITE', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_CLESECRETE', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_LIGHTWIDGET', '1', false, null, $this->current_shop_id);
            // simple stars by default
            Configuration::updateValue('AV_MULTILINGUE', '0', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_MULTISITE', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_PROCESSINIT', 'onorderstatuschange', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_ORDERSTATESCHOOSEN', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_RELOADED', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_ACTIVATE_COLLECT', false, false, null, $this->current_shop_id);
            Configuration::updateValue('AV_CLIENTID', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_SECRETAPI', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_DELAY', '0', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_DELAY_PRODUIT', '0', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_GETPRODREVIEWS', 'yes', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_DISPLAYPRODREVIEWS', '', false, null, $this->current_shop_id);
            Configuration::updateValue(
                'AV_CSVFILENAME',
                'Export_NetReviews_01-01-1970-default.csv',
                false,
                null,
                $this->current_shop_id
            );
            Configuration::updateValue('AV_SCRIPTFLOAT', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_SCRIPTFLOAT_ALLOWED', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_SCRIPTFIXE', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_SCRIPTFIXE_ALLOWED', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_GOUPINFO', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_URLCERTIFICAT', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_FORBIDDEN_EMAIL', 'marketplace.amazon.com;marketplace.amazon.fr;rueducommerce.com;laredoute.com;zalando.fr;priceminister.com;cdiscount.com;sc.rueducommerce.com;marketplace.fnac.com;clemarche.com;noreplylziflux.fr;marketplace.amazon.it;natureetdecouvertes.com;mp.laredoute-marketplace.fr;monechelle.com;galeries_lafayette.com;noreply-iziflux.com;marketplace.amazon.co.uk;anonyme.com;darty.fr;auchan.fr;marketplace.amazon.es;manomano.fr;noreplyiziflux.fr;alert-shopping-flux.com;macway.fr;downtownstock.com;roure.net;message.manomano.com;notification.mirakl.net', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_CODE_LANG', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_SNIPPETSITETYPE', '1', false, null, $this->current_shop_id);
            // Type of RS
            Configuration::updateValue('AV_DISPLAYSNIPPETSITE', '1', false, null, $this->current_shop_id);
            // RS enabled by defaut
            Configuration::updateValue('AV_RICHSNIPPETSWEBSITE', '1', false, null, $this->current_shop_id);
            // website has its own RS enabled by defaut
            Configuration::updateValue('AV_NBOFREVIEWS', '5', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_STARCOLOR', 'FFCD00', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_PRODUCTUNIGINFO', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_NBOPRODUCTS', '', false, null, $this->current_shop_id);
            if (version_compare(_PS_VERSION_, '1.7', '>')) {
                Configuration::updateValue('AV_EXTRA_OPTION', '2', false, null, $this->current_shop_id);
            // hookDisplayProductButtons
            } else {
                Configuration::updateValue('AV_EXTRA_OPTION', '0', false, null, $this->current_shop_id);
                // hookExtraright
            }
            Configuration::updateValue(
                'AV_DISPLAYSTARPLIST',
                '0',
                false,
                null,
                $this->current_shop_id
            );
            Configuration::updateValue('AV_TEMPLATE', '1', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_TABSHOW', '1', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_FORMAT_IMAGE', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_TABNEWNAME', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_STARSHOMESHOW', '1', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_NRESPONSIVE', '0', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_HELPFULHIDE', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_MEDIAHIDE', '', false, null, $this->current_shop_id);
            Configuration::updateValue(
                'AV_LIMIT_LOST_ORDERS',
                date('Y-m-d'),
                false,
                null,
                $this->current_shop_id
            );
            Configuration::updateValue('AV_RELOADED', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_SCRIPTTAGJS', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_AVERAGETAGJS', '', false, null, $this->current_shop_id);
            Configuration::updateValue('AV_REVIEWSTAGJS', '', false, null, $this->current_shop_id);
        }

        if (version_compare(_PS_VERSION_, '1.5', '>=')) {
            if (false == parent::install() ||
                !$this->registerHook('productTab') ||
                !$this->registerHook('productTabContent') ||
                !$this->registerHook('header') ||
                !$this->registerHook('footer') ||
                !$this->registerHook('displayHeader') ||
                !$this->registerHook('displayProductButtons') ||
                !$this->registerHook('displayProductPriceBlock') ||
                !$this->registerHook('displayRightColumnProduct') ||
                !$this->registerHook('displayLeftColumnProduct') ||
                !$this->registerHook('displayProductExtraContent') ||
                !$this->registerHook('displayBeforeBodyClosingTag') ||
                !$this->registerHook('displayProductListReviews') ||
                !$this->registerHook('displayRightColumn') ||
                !$this->registerHook('displayLeftColumn') ||
                !$this->registerHook('displayPaymentTop') ||
                !$this->registerHook('ExtraNetreviews') ||
                !$this->registerHook('TabcontentNetreviews') ||
                !$this->registerHook('GlobalnoteNetreviews') ||
                !$this->registerHook('CategorystarsNetreviews') ||
                !$this->registerHook('CategorysummaryNetreviews') ||
                !$this->registerHook('actionOrderStatusPostUpdate') ||
                !$this->registerHook('actionValidateOrder') ||
                !$this->registerHook('displayFooter')) {
                return false;
            } else {
                return true;
            }
        } else {
            if (false == parent::install() ||
                !$this->registerHook('productTab') ||
                !$this->registerHook('productTabContent') ||
                !$this->registerHook('header') ||
                !$this->registerHook('footer') ||
                !$this->registerHook('rightColumn') ||
                !$this->registerHook('leftColumn') ||
                !$this->registerHook('extraRight') ||
                !$this->registerHook('extraLeft') ||
                !$this->registerHook('OrderConfirmation') ||
                !$this->registerHook('updateOrderStatus')) {
                return false;
            } else {
                return true;
            }
        }
    }

    public function uninstall($keep = true)
    {
        $sql = 'SELECT name FROM ' . _DB_PREFIX_ . "configuration where name like 'AV_%'";
        if ($results = Db::getInstance()->ExecuteS($sql)) {
            foreach ($results as $row) {
                Configuration::deleteByName($row['name']);
            }
        }

        if (!parent::uninstall() || ($keep && !$this->deleteTables()) ||
            !$this->unregisterHook('productTab') ||
            !$this->unregisterHook('productTabContent') ||
            !$this->unregisterHook('displayHeader') ||
            !$this->unregisterHook('header') ||
            !$this->unregisterHook('footer') ||
            !$this->unregisterHook('displayProductButtons') ||
            !$this->unregisterHook('displayProductPriceBlock') ||
            !$this->unregisterHook('displayRightColumnProduct') ||
            !$this->unregisterHook('displayLeftColumnProduct') ||
            !$this->unregisterHook('displayProductExtraContent') ||
            !$this->unregisterHook('displayBeforeBodyClosingTag') ||
            !$this->unregisterHook('displayProductListReviews') ||
            !$this->unregisterHook('displayRightColumn') ||
            !$this->unregisterHook('displayLeftColumn') ||
            !$this->unregisterHook('displayPaymentTop') ||
            !$this->unregisterHook('ExtraNetreviews') ||
            !$this->unregisterHook('TabcontentNetreviews') ||
            !$this->unregisterHook('GlobalnoteNetreviews') ||
            !$this->unregisterHook('CategorystarsNetreviews') ||
            !$this->unregisterHook('CategorysummaryNetreviews') ||
            !$this->unregisterHook('actionOrderStatusPostUpdate') ||
            !$this->unregisterHook('actionValidateOrder') ||
            !$this->unregisterHook('displayFooter')) {
            return false;
        }

        return true;
    }

    public function createTables()
    {
        $sql = [];
        $sql[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'av_products_reviews;';
        $sql[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'av_products_average;';
        $sql[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'av_orders;';
        $sql[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . 'av_products_reviews (
                      `id_product_av` varchar(36) NOT NULL,
                      `ref_product` varchar(20) NOT NULL,
                      `rate` varchar(5) NOT NULL,
                      `review` text NOT NULL,
                      `customer_name` varchar(30) NOT NULL,
                      `horodate` text NOT NULL,
                      `horodate_order` text NOT NULL,
                      `discussion` text NULL,
                      `helpful` int(7) DEFAULT 0,
                      `helpless` int(7) DEFAULT 0,
                      `media_full` text NULL,
                      `iso_lang` varchar(5) DEFAULT "0",
                      `id_shop` int(2) DEFAULT 0,
                      PRIMARY KEY (`id_product_av`,`iso_lang`,`id_shop`)
                    ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';
        $sql[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . 'av_products_average (
                      `id_product_av` varchar(36) NOT NULL,
                      `ref_product` varchar(20) NOT NULL,
                      `rate` varchar(5) NOT NULL,
                      `nb_reviews` int(10) NOT NULL,
                      `horodate_update` text NOT NULL,
                      `iso_lang` varchar(5) DEFAULT "0",
                      `id_shop` int(2) DEFAULT 0,
                      PRIMARY KEY (`ref_product`,`iso_lang`,`id_shop`)
                    ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';
        $sql[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . 'av_orders (
                      `id_order` int(11) NOT NULL,
                      `id_shop` int(2) DEFAULT 0,
                      `flag_get` int(2) DEFAULT NULL,
                      `horodate_get` varchar(25) DEFAULT NULL,
                      `id_order_state` int(5) DEFAULT NULL,
                      `iso_lang` varchar(5) DEFAULT "0",
                      `horodate_now` timestamp NULL DEFAULT CURRENT_TIMESTAMP,
                      PRIMARY KEY (`id_order`,`iso_lang`,`id_shop`)
                    ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';

        foreach ($sql as $query) {
            if (false == Db::getInstance()->execute($query)) {
                return $query;
            }
        }
    }

    public function reset()
    {
        if (!$this->uninstall(false)) {
            return false;
        }
        if (!$this->install(false)) {
            return false;
        }

        return true;
    }

    public function deleteTables()
    {
        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS
            `' . _DB_PREFIX_ . 'av_orders`,
            `' . _DB_PREFIX_ . 'av_products_reviews`,
            `' . _DB_PREFIX_ . 'av_products_average`');
    }

    /**
     * Load the configuration form.
     */
    public function getContent()
    {
        if (version_compare(_PS_VERSION_, '1.6', '<')) {
            $this->addFiles('avisverifies-style-back-old', 'css');
        }
        $this->addFiles('avisverifies-admin', 'css');

        if (!empty($_POST)) {
            $this->postProcess();
        }

        if (1 == Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE') &&
            (Shop::CONTEXT_ALL == Shop::getContext() ||
                Shop::CONTEXT_GROUP == Shop::getContext())) {
            $this->html .= $this->displayError(
                $this->l('Multistore feature is enabled. 
                    Please choose above the store to configure.')
            );

            return $this->html;
        }

        $o_av = new NetReviewsModel();
        $nb_reviews = $o_av->getTotalReviews();
        $nb_reviews_average = $o_av->getTotalReviewsAverage();
        $nb_orders = $o_av->getTotalOrders();
        $current_avisverifies_idwebsite = [];
        $current_avisverifies_clesecrete = [];
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $employee_lang = $this->context->employee->id_lang;
            $order_statut_list = OrderState::getOrderStates((int) $employee_lang);
        } else {
            $order_statut_list = OrderState::getOrderStates((int) Configuration::get('PS_LANG_DEFAULT'));
        }

        $multisite = Configuration::get('AV_MULTISITE');
        if (!empty($multisite)) {
            $idshop = $this->current_shop_id;
        } else {
            $idshop = null;  // if multishop but only one shop enabled or non multishop
        }

        if (_PS_VERSION_ >= '1.5') {
            $isMultishop = Shop::isFeatureActive();
            if ($isMultishop && Shop::getTotalShops() > 1 && _PS_VERSION_ >= '1.5'
                /* && _PS_VERSION_ < '1.7' */ && (Configuration::hasKey(
                    'AV_IDWEBSITE',
                    null,
                    null,
                    null
                ) || Configuration::hasKey('AV_GOUPINFO', null, null, null))) {
                $multisite = Configuration::get('AV_MULTISITE', null, null, null);
                $idWebsiteShop1 = Configuration::get('AV_IDWEBSITE', null, null, null);
                $secretKeyShop1 = Configuration::get('AV_CLESECRETE', null, null, null);
                $groupInfoShop1 = Configuration::get('AV_GOUPINFO', null, null, null);

                Configuration::deleteByName('AV_IDWEBSITE');
                Configuration::updateValue('AV_IDWEBSITE', $idWebsiteShop1, null, $this->current_shop_id);
                Configuration::deleteByName('AV_CLESECRETE');
                Configuration::updateValue('AV_CLESECRETE', $secretKeyShop1, null, $this->current_shop_id);
                Configuration::deleteByName('AV_GOUPINFO');
                Configuration::updateValue('AV_GOUPINFO', $groupInfoShop1, null, $this->current_shop_id);
                Configuration::deleteByName('AV_MULTISITE');
                Configuration::updateValue('AV_MULTISITE', $multisite, null, $this->current_shop_id);
            }
        }

        $current_avisverifies_idwebsite['root'] = Configuration::get(
            'AV_IDWEBSITE',
            null,
            null,
            $this->current_shop_id
        );
        $current_avisverifies_clesecrete['root'] = Configuration::get(
            'AV_CLESECRETE',
            null,
            null,
            $this->current_shop_id
        );

        $languages = Language::getLanguages(true);
        foreach ($languages as $lang) {
            $current_avisverifies_idwebsite[$lang['iso_code']] = '';
            $current_avisverifies_clesecrete[$lang['iso_code']] = '';
            $language_group_name = $this->getIdConfigurationGroup($lang['iso_code']);
            if (!Configuration::get('AV_IDWEBSITE' . $language_group_name, null, null, $this->current_shop_id)) {
                Configuration::updateValue(
                    'AV_IDWEBSITE' . $language_group_name,
                    '',
                    false,
                    null,
                    $this->current_shop_id
                );
            } elseif ($language_group_name) {
                $current_avisverifies_idwebsite[$lang['iso_code']] = Configuration::get(
                    'AV_IDWEBSITE' . $language_group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
            }

            if (!Configuration::get('AV_CLESECRETE' .
             $language_group_name, null, null, $this->current_shop_id)) {
                Configuration::updateValue(
                    'AV_CLESECRETE' . $language_group_name,
                    '',
                    false,
                    null,
                    $this->current_shop_id
                );
            } elseif ($language_group_name) {
                $current_avisverifies_clesecrete[$lang['iso_code']] = Configuration::get(
                    'AV_CLESECRETE' . $language_group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
            }
        }

        // update av_groupinfo
        Configuration::updateValue('AV_GOUPINFO', NetReviewsModel::avJsonEncode($current_avisverifies_idwebsite), false, null, $this->current_shop_id);
        $keycheck = NetReviewsModel::getMultiShopValues('AV_GOUPINFO');
        $keycheck = ($keycheck) ? $keycheck : [];
        $keycheck_result = [];
        $keycheck = NetReviewsModel::avJsonEncode(array_map('json_decode', $keycheck));
        $keycheck = json_decode($keycheck, true);

        foreach ($keycheck as $value_keycheck) {
            if ($value_keycheck) {
                foreach ($value_keycheck as $lang_check => $value) {
                    if ('root' != $lang_check && '' != $value) {
                        $keycheck_result['lan'][] = $value;
                    } elseif ('root' == $lang_check && '' != $value) {
                        $keycheck_result['root'][] = $value;
                    }
                }
            }
        }

        $double_key_found = false;
        if (isset($keycheck_result['lan']) && (count($keycheck_result['lan']) !== count(
            array_unique($keycheck_result['lan'])
        ))) {
            $double_key_found = true;
        } elseif (isset($keycheck_result['root']) && (count($keycheck_result['root']) !== count(
            array_unique($keycheck_result['root'])
        ))) {
            $double_key_found = true;
        }

        if (!empty($keycheck_result) && $double_key_found) {
            $this->html .= $this->displayError($this->l('Each idWebsite and key can be used only once, 
                pleack check your current informations'));
            Configuration::deleteFromContext('AV_GOUPINFO');
            Configuration::deleteFromContext('AV_CLESECRETE');
            Configuration::deleteFromContext('AV_IDWEBSITE');
            $current_avisverifies_idwebsite['root'] = Configuration::get(
                'AV_IDWEBSITE',
                null,
                null,
                $this->current_shop_id
            );
            $current_avisverifies_clesecrete['root'] = Configuration::get(
                'AV_CLESECRETE',
                null,
                null,
                $this->current_shop_id
            );
        }

        if (version_compare(_PS_VERSION_, '1.5', '<') && version_compare(_PS_VERSION_, '1.4', '>=')) {
            global $currentIndex;
            $url_back = $currentIndex . '&configure=netreviews&token=' .
            Tools::getAdminTokenLite('AdminModules') .
                '&tab_module=advertising_marketing&module_name=netreviews';
        } elseif (version_compare(_PS_VERSION_, '1.5', '>=')) {
            $url_back = ($this->context->link->getAdminLink('AdminModules') .
                '&configure=' . $this->name . '&tab_module=' .
                $this->tab . '&conf=4&module_name=' . $this->name);
        } else {
            $url_back = '';
        }
        $use_image = false;
        $use_star_format_image = Configuration::get('AV_FORMAT_IMAGE', null, null, $idshop);
        if (version_compare(_PS_VERSION_, '1.4', '>=') && '1' != $use_star_format_image) {
            $stars_file = 'avisverifies-stars-font.tpl';
        } else {
            $stars_file = 'avisverifies-stars-image.tpl';
            $use_image = true;
        }
        $demo_rate = 4.5;
        $average_rate_percent = [];
        $average_rate_percent['floor'] = floor($demo_rate) - 1;
        $average_rate_percent['decimals'] = ($demo_rate - floor($demo_rate)) * 20;
        $customized_star_color = (Configuration::get('AV_STARCOLOR', null, null, $idshop)) ?
            Configuration::get('AV_STARCOLOR', null, null, $idshop) : 'FFCD00'; // default #FFCD00
        if (version_compare(_PS_VERSION_, '1.6.1', '>=') || method_exists($this, 'getPossibleHooksList')) {
            $hook_list = $this->getPossibleHooksList();
        } else {
            if (version_compare(_PS_VERSION_, '1.5', '<')) {
                $currentidshop = null;
            } else {
                $currentidshop = $this->current_shop_id;
            }
            $hook_list = $o_av->listRegisteredHooks($this->id, $currentidshop);
        }

        $installationModuleDate = date('Y-m-d', strtotime(Configuration::get('AV_LIMIT_LOST_ORDERS')));
        $today = date('Y-m-d');

        $this->smartyAssign([
            'base_url' => __PS_BASE_URI__,
            'current_avisverifies_urlapi' => Configuration::get(
                'AV_URLAPI',
                null,
                null,
                $this->current_shop_id
            ),
            'current_lightwidget_checked' => Configuration::get(
                'AV_LIGHTWIDGET',
                null,
                null,
                $this->current_shop_id
            ),
            'current_multilingue_checked' => Configuration::get(
                'AV_MULTILINGUE',
                null,
                null,
                $this->current_shop_id
            ),
            'current_starproductlist_checked' => Configuration::get(
                'AV_DISPLAYSTARPLIST',
                null,
                null,
                $this->current_shop_id
            ),
            'current_template_checked' => Configuration::get(
                'AV_TEMPLATE',
                null,
                null,
                $this->current_shop_id
            ),
            'current_snippets_website_global_checked' => Configuration::get(
                'AV_SNIPPETSITETYPE',
                null,
                null,
                $this->current_shop_id
            ),
            'current_snippets_site_checked' => Configuration::get(
                'AV_DISPLAYSNIPPETSITE',
                null,
                null,
                $this->current_shop_id
            ),
            'richSnippetsWebsite_checked' => Configuration::get(
                'AV_RICHSNIPPETSWEBSITE',
                null,
                null,
                $this->current_shop_id
            ),
            'avisverifies_nb_reviews' => Configuration::get(
                'AV_NBOFREVIEWS',
                null,
                null,
                $this->current_shop_id
            ),
            'avisverifies_stars_custom_color' => Configuration::get(
                'AV_STARCOLOR',
                null,
                null,
                $this->current_shop_id
            ),
            'productuniqueginfo_checked' => Configuration::get(
                'AV_PRODUCTUNIGINFO',
                null,
                null,
                $this->current_shop_id
            ),
            'customized_star_color' => $customized_star_color,
            'avisverifies_nb_products' => Configuration::get(
                'AV_NBOPRODUCTS',
                null,
                null,
                $this->current_shop_id
            ),
            'avisverifies_extra_option' => Configuration::get(
                'AV_EXTRA_OPTION',
                null,
                null,
                $this->current_shop_id
            ),
            'current_nresponsive_checked' => Configuration::get(
                'AV_NRESPONSIVE',
                null,
                null,
                $this->current_shop_id
            ),
            'current_hidehelpful_checked' => Configuration::get(
                'AV_HELPFULHIDE',
                null,
                null,
                $this->current_shop_id
            ),
            'current_hidemedia_checked' => Configuration::get(
                'AV_MEDIAHIDE',
                null,
                null,
                $this->current_shop_id
            ),
            'avisverifies_rename_tag' => Configuration::get(
                'AV_TABNEWNAME',
                null,
                null,
                $this->current_shop_id
            ),
            'tabshow_checked' => Configuration::get(
                'AV_TABSHOW',
                null,
                null,
                $this->current_shop_id
            ),
            'stars_image' => Configuration::get(
                'AV_FORMAT_IMAGE',
                null,
                null,
                $this->current_shop_id
            ),
            'starshome_checked' => Configuration::get(
                'AV_STARSHOMESHOW',
                null,
                null,
                $this->current_shop_id
            ),
            'av_reloaded' => Configuration::get(
                'AV_RELOADED',
                null,
                null,
                $this->current_shop_id
            ),
            'current_avisverifies_idwebsite' => $current_avisverifies_idwebsite,
            'current_avisverifies_clesecrete' => $current_avisverifies_clesecrete,
            'version' => $this->version,
            'version_ps' => _PS_VERSION_,
            'order_statut_list' => $order_statut_list,
            'languages' => $languages,
            'debug_nb_reviews' => $nb_reviews['nb_reviews'],
            'debug_nb_reviews_average' => $nb_reviews_average['nb_reviews_average'],
            'debug_nb_orders_flagged' => $nb_orders['flagged']['nb'],
            'debug_nb_orders_not_flagged' => $nb_orders['not_flagged']['nb'],
            'debug_nb_orders_all' => $nb_orders['all']['nb'],
            'av_path' => $this->_path,
            'shop_name' => Configuration::get('PS_SHOP_NAME'),
            // (!empty($this->context->shop->name))?$this->context->shop->name:"",
            'url_back' => $url_back,
            'stars_dir' => _PS_ROOT_DIR_ . '/modules/netreviews/views/templates/hook/sub/' . $stars_file,
            'use_image' => $use_image,
            'average_rate_percent' => $average_rate_percent,
            'av_rate_percent_int' => ($demo_rate) ? round($demo_rate * 20) : 100,
            'hook_list' => $hook_list,
            'installationModuleDate' => $installationModuleDate,
            'today' => $today,
            'collect_consent' => Configuration::get('AV_COLLECT_CONSENT', null, null, $this->current_shop_id),
        ]);

        $tpl = 'avisverifies-backoffice';
        $this->html .= $this->displayTemplate($tpl);

        return $this->html;
    }

    /**
     * Save configuration form.
     */
    protected function postProcess()
    {
        if (Tools::isSubmit('submit_export')) {
            try {
                $o_av = new NetReviewsModel();
                $header_colums = 'id_order;reference;order_amount;email;firstname;lastname;date_order;payment_method;
                carrer;delay;id_product;category;description;ean13;upc;mpn;brand;product_url;image_product_url;
                order_state_id;order_state;iso_lang;id_shop' . "\r\n";

                $return_export = $o_av->export($header_colums, $this->current_shop_id);
                if (file_exists($return_export[2])) {
                    $this->html .= $this->displayConfirmation(sprintf(
                        $this->l('%s orders have been exported.'),
                        $return_export[1]
                    ) . '<a href="../modules/netreviews/Export_NetReviews_' . $return_export[0] . '"> 
                    ' . $this->l('Click here to download the file') . '</a>');
                } else {
                    $this->html .= $this->displayError($this->l('Writing on the server is not allowed. Please assign 
                        write permissions to the folder netreviews') . $return_export[2]);
                }
            } catch (Exception $e) {
                $this->html .= $this->displayError($e->getMessage());
            }
        }

        if (Tools::isSubmit('submit_configuration')) {
            $id_website_current = trim(Tools::getValue('avisverifies_idwebsite'));
            $cle_secrete_current = trim(Tools::getValue('avisverifies_clesecrete'));

            Configuration::updateValue(
                'AV_MULTILINGUE',
                Tools::getValue('avisverifies_multilingue'),
                false,
                null,
                $this->current_shop_id
            );
            Configuration::updateValue('AV_IDWEBSITE', $id_website_current, false, null, $this->current_shop_id);
            Configuration::updateValue('AV_CLESECRETE', $cle_secrete_current, false, null, $this->current_shop_id);
            Configuration::updateValue('AV_MULTISITE', $this->current_shop_id);

            if ('checked' == Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id)) {
                if (version_compare(_PS_VERSION_, '1.5', '>=')) {
                    $sql = '
                    SELECT name FROM ' . _DB_PREFIX_ . "configuration
                    where (name like 'AV_GROUP_CONF_%'
                    OR name like 'AV_IDWEBSITE_%'
                    OR name like 'AV_CLESECRETE_%')
                    AND id_shop = '" . $this->current_shop_id . "'
                    ";
                } else {
                    $sql = '
                    SELECT name FROM ' . _DB_PREFIX_ . "configuration
                    where (name like 'AV_GROUP_CONF_%'
                    OR name like 'AV_IDWEBSITE_%'
                    OR name like 'AV_CLESECRETE_%')
                    ";
                }

                $idshop_conf = true;
                if ($results = Db::getInstance()->ExecuteS($sql)) {
                    foreach ($results as $row) {
                        if (version_compare(_PS_VERSION_, '1.5', '>=')) {
                            Configuration::deleteFromContext($row['name']);
                        } else {
                            Configuration::deleteByName($row['name']);
                        }
                    }
                } else {
                    $idshop_conf = false; // if multishop but only one shop enabled or non multishop
                    $sql_without_idshop = '
                            SELECT name FROM ' . _DB_PREFIX_ . "configuration
                            where (name like 'AV_GROUP_CONF_%'
                            OR name like 'AV_IDWEBSITE_%'
                            OR name like 'AV_CLESECRETE_%')";
                    if ($results = Db::getInstance()->ExecuteS($sql_without_idshop)) {
                        foreach ($results as $row) {
                            if (version_compare(_PS_VERSION_, '1.5', '>=')) {
                                Configuration::deleteFromContext($row['name']);
                            } else {
                                Configuration::deleteByName($row['name']);
                            }
                        }
                    }
                }
                Configuration::updateValue('AV_MULTISITE', $idshop_conf);
                // in case that it's not multishop while configurated as multishop
                $languages = Language::getLanguages(true);
                $this->setIdConfigurationGroup($languages, $idshop_conf);
            }
        }

        if (Tools::isSubmit('submit_advanced')) {
            $keys = [
                'avisverifies_lightwidget' => 'AV_LIGHTWIDGET',
                'netreviews_snippets_site' => 'AV_DISPLAYSNIPPETSITE',
                'netreviews_snippets_website_global' => 'AV_SNIPPETSITETYPE',
                'avisverifies_checkRichSnippetsWebsite_show' => 'AV_RICHSNIPPETSWEBSITE',
                'avisverifies_nb_reviews' => 'AV_NBOFREVIEWS',
                'avisverifies_stars_custom_color' => 'AV_STARCOLOR',
                'avisverifies_productuniqueginfo' => 'AV_PRODUCTUNIGINFO',
                'avisverifies_nb_products' => 'AV_NBOPRODUCTS',
                'avisverifies_rename_tag' => 'AV_TABNEWNAME',
                'avisverifies_extra_option' => 'AV_EXTRA_OPTION',
                'avisverifies_star_productlist' => 'AV_DISPLAYSTARPLIST',
                'avisverifies_template' => 'AV_TEMPLATE',
                'avisverifies_tab_show' => 'AV_TABSHOW',
                'avisverifies_stars_image' => 'AV_FORMAT_IMAGE',
                'avisverifies_starshome_show' => 'AV_STARSHOMESHOW',
                'avisverifies_hidehelpful' => 'AV_HELPFULHIDE',
                'avisverifies_hidemedia' => 'AV_MEDIAHIDE',
                'avisverifies_nresponsive' => 'AV_NRESPONSIVE',
            ];

            foreach ($keys as $index => $key) {
                if (null != Tools::getValue($index)) {
                    Configuration::updateValue(
                        $key,
                        Tools::getValue($index),
                        false,
                        null,
                        $this->current_shop_id
                    );
                }
            }
        }

        if (Tools::isSubmit('submit_addhooklist')) {
            if (!$this->registerHook('productTab') ||
                !$this->registerHook('displayHeader') ||
                !$this->registerHook('productTabContent') ||
                !$this->registerHook('header') ||
                !$this->registerHook('footer') ||
                !$this->registerHook('displayProductButtons') ||
                !$this->registerHook('displayProductPriceBlock') ||
                !$this->registerHook('displayRightColumnProduct') ||
                !$this->registerHook('displayLeftColumnProduct') ||
                !$this->registerHook('displayProductExtraContent') ||
                !$this->registerHook('displayBeforeBodyClosingTag') ||
                !$this->registerHook('displayProductListReviews') ||
                !$this->registerHook('displayRightColumn') ||
                !$this->registerHook('displayLeftColumn') ||
                !$this->registerHook('displayPaymentTop') ||
                !$this->registerHook('ExtraNetreviews') ||
                !$this->registerHook('TabcontentNetreviews') ||
                !$this->registerHook('GlobalnoteNetreviews') ||
                !$this->registerHook('CategorystarsNetreviews') ||
                !$this->registerHook('CategorysummaryNetreviews') ||
                !$this->registerHook('actionOrderStatusPostUpdate') ||
                !$this->registerHook('actionValidateOrder') ||
                !$this->registerHook('displayFooter')) {
                return false;
            } else {
                return true;
            }
        }

        if (Tools::isSubmit('submit_purge')) {
            $query_id_shop = '';
            if (1 == Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
                $query_id_shop = ' AND oav.id_shop = ' . (int) $this->current_shop_id;
            }

            $query = '  SELECT oav.id_order, o.date_add as date_order,o.id_customer
                        FROM ' . _DB_PREFIX_ . 'av_orders oav
                        LEFT JOIN ' . _DB_PREFIX_ . 'orders o
                        ON oav.id_order = o.id_order
                        LEFT JOIN ' . _DB_PREFIX_ . 'order_history oh
                        ON oh.id_order = o.id_order
                        WHERE (oav.flag_get IS NULL OR oav.flag_get = 0)'
                . $query_id_shop;

            $orders_list = Db::getInstance()->ExecuteS($query);
            if (!empty($orders_list)) {
                foreach ($orders_list as $order) { /* Set orders as getted */
                    Db::getInstance()->Execute('UPDATE ' . _DB_PREFIX_ . 'av_orders
                                                SET horodate_get = "' . time() . '", flag_get = 1
                                                WHERE id_order = ' . (int) $order['id_order']);
                }
                $this->html .= $this->displayConfirmation(sprintf(
                    $this->l('The orders has been purged for %s'),
                    $this->context->shop->name
                ));
            } else {
                $this->html .= $this->displayError(sprintf(
                    $this->l('No orders to purged for %s'),
                    $this->context->shop->name
                ));
            }
        }

        if (Tools::isSubmit('submit_generateLostOrders_period') ||
            Tools::isSubmit('submit_generateLostOrders_all')) {
            if (!empty(Tools::getValue('submit_generateLostOrders_end'))) {
                $date = Tools::getValue('submit_generateLostOrders_end');
                $date = (new DateTime($date))->add(new DateInterval('P1D'));
                $endDate = $date->format('Y-m-d');

                if (!empty(Tools::getValue('submit_generateLostOrders_start'))) {
                    $startDate = Tools::getValue('submit_generateLostOrders_start');
                } else {
                    $startDate = Configuration::get('AV_LIMIT_LOST_ORDERS');
                }
                $dateForQuery = ' AND o.date_add >= "' . $startDate .
                '" AND o.date_add < "' . $endDate . '"';
            } elseif (!empty(Tools::getValue('submit_generateLostOrders_all'))) {
                if (1 == Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
                    $id_shop = (int) $this->current_shop_id;
                    if (!Configuration::hasKey('AV_LIMIT_LOST_ORDERS', null, null, $id_shop)) {
                        Configuration::updateValue(
                            'AV_LIMIT_LOST_ORDERS',
                            date('Y-m-d'),
                            false,
                            null,
                            $id_shop
                        );
                    } else {
                        $startDate = Configuration::get('AV_LIMIT_LOST_ORDERS');
                    }
                } else {
                    if (version_compare(_PS_VERSION_, '1.5', '<')) {
                        if (false == Configuration::get('AV_LIMIT_LOST_ORDERS')) {
                            Configuration::updateValue('AV_LIMIT_LOST_ORDERS', date('Y-m-d'));
                        }
                    } else {
                        if (!Configuration::hasKey('AV_LIMIT_LOST_ORDERS')) {
                            Configuration::updateValue('AV_LIMIT_LOST_ORDERS', date('Y-m-d'));
                        }
                    }
                    $startDate = Configuration::get('AV_LIMIT_LOST_ORDERS');
                }
                $dateForQuery = ' AND o.date_add > "' . $startDate . '"';
            }

            $query_id_shop = '';
            if (1 == Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
                $query_id_shop = ' AND o.id_shop = ' . (int) $this->current_shop_id;
            }

            // On récupère toutes les commandes créées depuis l'installation
            // du module et qui ne sont pas dans notre table
            if (version_compare(_PS_VERSION_, '1.5', '>=')) {
                $query = 'SELECT o.id_order, o.date_add as date_order, o.id_shop, lg.iso_code
                        FROM ' . _DB_PREFIX_ . 'orders o
                        LEFT JOIN ' . _DB_PREFIX_ . 'lang lg
                            ON o.id_lang = lg.id_lang
                        LEFT JOIN ' . _DB_PREFIX_ . 'av_orders oav
                            ON o.id_order = oav.id_order
                        WHERE o.id_order NOT IN (SELECT oav.id_order FROM ' . _DB_PREFIX_ . 'av_orders oav )' .
                    $dateForQuery . $query_id_shop;
            } else {
                $query = 'SELECT o.id_order, o.date_add as date_order, lg.iso_code
                        FROM ' . _DB_PREFIX_ . 'orders o
                        LEFT JOIN ' . _DB_PREFIX_ . 'lang lg
                            ON o.id_lang = lg.id_lang
                        LEFT JOIN ' . _DB_PREFIX_ . 'av_orders oav
                            ON o.id_order = oav.id_order
                        WHERE o.id_order NOT IN (SELECT oav.id_order FROM ' . _DB_PREFIX_ . 'av_orders oav )' .
                    $dateForQuery;
            }

            $orders_list = Db::getInstance()->ExecuteS($query);
            $i = 0;

            if (version_compare(_PS_VERSION_, '1.5', '<')) {
                $shopName = Configuration::get('PS_SHOP_NAME');
            } else {
                $shopName = $this->context->shop->name;
            }

            if (!empty($orders_list)) {
                foreach ($orders_list as $order) {
                    if (version_compare(_PS_VERSION_, '1.5', '>=')) {
                        $qry_order_insert = '
                        INSERT INTO ' . _DB_PREFIX_ . 'av_orders (id_order, id_shop, iso_lang, flag_get, 
                        horodate_now) VALUES (' . $order['id_order'] . ',"' . $order['id_shop'] . '","' .
                            $order['iso_code'] . '", 0 ,"' . pSQL($order['date_order']) . '")
                            ';
                    } else {
                        $qry_order_insert = '
                        INSERT INTO ' . _DB_PREFIX_ . 'av_orders (id_order, iso_lang, flag_get, 
                        horodate_now) 
                        VALUES (' . $order['id_order'] . ',"' . $order['iso_code'] . '", 0 ,"' .
                            pSQL($order['date_order']) . '")
                            ';
                    }
                    $result = Db::getInstance()->Execute($qry_order_insert);
                    if ($result) {
                        ++$i;
                    } else {
                        $this->html .= $this->displayError(sprintf($this->l('There was a problem. 
                            Check MySQL errors')));

                        return false;
                    }
                }
                $this->html .= $this->displayConfirmation(sprintf($this->l($i .
                    ' orders have been added to be collected for %s'), $shopName));
            } else {
                $this->html .= $this->displayError(sprintf($this->l('No orders to add for %s'), $shopName));
            }
        }
    }

    /**
     * Allow to add the widget flottant code to the hook header in front office if configurated.
     *
     * Return widget flottant code if configurated
     *
     * @return javascript string in hook header
     */
    public function hookDisplayHeader($params)
    {
        $isTagJS = false;
        $av_template = Configuration::get('AV_TEMPLATE', null, null, $this->current_shop_id);
        if ($av_template && '2' == $av_template) {
            $css = 'avisverifies-template2';
        } else {
            $css = 'avisverifies-template1';
        }
        // ************************************ TagJS *****************************************************************************************
        if ($this->storeKeysConfigured()) {   // Test if Store / Storefront (multilange) has idWebsite & secretKey
            if (Configuration::get('AV_RELOADED' . $this->group_name, null, null, $this->current_shop_id)) {
                $isTagJS = true;
                // TagJS css specifically for category page
                if (property_exists($this->context->controller, 'php_self') && in_array($this->context->controller->php_self, ['index', 'category'])) {
                    $this->addFiles('category-tagjs', 'css');
                }
            }
        }
        // ***************************** End TagJS *********************************************************************************************
        if ($isTagJS) {                   // home page, product page, categoy page - TagJS script pages do not add module css/js
            if (property_exists($this->context->controller, 'php_self') && !in_array($this->context->controller->php_self, ['index', 'product', 'category'])) {
                $this->addFiles($css, 'css');
                $this->addFiles('avisverifies-tpl', 'js');
            }
        } else {
            $this->addFiles($css, 'css');
            $this->addFiles('avisverifies-tpl', 'js');
        }
        if ('checked' === Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id)) {
            if (null !== $this->group_name) {
                $avisverifies_scriptfloat_allowed = Configuration::get(
                    'AV_SCRIPTFLOAT_ALLOWED' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
                $av_scriptflottant = Configuration::get(
                    'AV_SCRIPTFLOAT' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
            } else {
                $avisverifies_scriptfloat_allowed = null;
                $av_scriptflottant = null;
            }
        } else {
            $avisverifies_scriptfloat_allowed = Configuration::get(
                'AV_SCRIPTFLOAT_ALLOWED',
                null,
                null,
                $this->current_shop_id
            );
            $av_scriptflottant = Configuration::get('AV_SCRIPTFLOAT', null, null, $this->current_shop_id);
        }

        $widget_flottant_code = '';
        if ((true != strpos(Tools::strtolower($av_scriptflottant), 'null') || Tools::strlen($av_scriptflottant) > 10) &&
            'yes' === $avisverifies_scriptfloat_allowed) {
            $widget_flottant_code .= "\n" . Tools::stripslashes(html_entity_decode($av_scriptflottant));
        }

        return $widget_flottant_code;
    }

    /**
     * display category stars TagJS.
     */
    public function categoryStarsTagJS($id_product)
    {
        $av_averagetagjs = configuration::get('AV_AVERAGETAGJS' . $this->group_name, null, null, $this->current_shop_id);
        $av_averagetagjs = Tools::stripslashes(html_entity_decode($av_averagetagjs));
        $av_script = configuration::get('AV_SCRIPTTAGJS' . $this->group_name, null, null, $this->current_shop_id);
        $avisverifies_display_stars = Configuration::get('AV_DISPLAYSTARPLIST', null, null, $this->current_shop_id);
        // Please note TagJS requires following variables, AverageTagJS, ScriptTagJS
        if (!empty($av_averagetagjs) && !empty($av_script) && !empty($avisverifies_display_stars)) {
            $this->context->smarty->assign('product_id', $id_product);
            try {
                // https://www.smarty.net/forums/viewtopic.php?p=94650&sid=81185d158408ef1d636d6a7dcc20c55c
                $av_averagetagjs = $this->context->smarty->fetch('string: ' . $av_averagetagjs);

                return "\n" . $av_averagetagjs;
            } catch (Exception $e) {
                return;
            }
        } else {
            // Then store / storefront is active for TagJS but has either chosen no to display reviews or is missing configuration variables (Tag or Script or both)
            return;
        }
    }

    /**
     * Integration stars on category page used 3 hooks,
     * hookCategorystarsNetreviews  (parent hook)
     * hookDisplayProductListReviews
     * hookDisplayProductPriceBlock @params[type]=before_price.
     */
    public function hookCategorystarsNetreviews($params)
    {
        $show_stars_home = Configuration::get(
            'AV_STARSHOMESHOW',
            null,
            null,
            $this->current_shop_id
        ); // 1 or null in defaut
        if ((property_exists($this->context->controller, 'php_self') && 'index' != $this->context->controller->php_self)
            || 0 != $show_stars_home) {
            if (is_object($params['product'])) {
                $id_product = (int) $params['product']->id_product;
                if (isset($params['product']->link)) {
                    $link = $params['product']->link;
                }
            } else {
                $id_product = (int) $params['product']['id_product'];
                if (isset($params['product']['link'])) {
                    $link = $params['product']['link'];
                }
            }
            $avisverifies_display_stars = Configuration::get('AV_DISPLAYSTARPLIST', null, null, $this->current_shop_id);
            if (!isset($id_product) || empty($id_product) || !isset($avisverifies_display_stars)
                || empty($avisverifies_display_stars)) {
                return;
            }
            // ************************************ Display TagJS *****************************************************************************************
            if ($this->storeKeysConfigured()) {   // Test if Store / Storefront (multilange) has idWebsite & secretKey
                if (Configuration::get('AV_RELOADED' . $this->group_name, null, null, $this->current_shop_id)) {
                    return $this->categoryStarsTagJS($id_product);
                }
            }
            // ***************************** End TagJS *****************************************************************************************************
            $o_av = new NetReviewsModel();

            $stats_product = (!isset($this->stats_product) || empty($this->stats_product)) ?
                $o_av->getStatsProduct($id_product, $this->group_name, $this->current_shop_id) : $this->stats_product;

            if ($this->all_multishops_reviews || $this->all_languages_reviews) { // override rate & nb_reviews
                $language_group = ($this->all_languages_reviews) ? null : $this->group_name;
                $shop_id_group = ($this->all_multishops_reviews) ? null : $this->current_shop_id;
                $reviews = $o_av->getProductReviews(
                    $id_product,
                    $language_group,
                    $shop_id_group,
                    0,
                    1,
                    'horodate_DESC',
                    0,
                    false
                );
                $num_reviews = count($reviews);
                if ($num_reviews < 1 || !$avisverifies_display_stars) {
                    return;
                }
                $somme_rate = 0;
                foreach ($reviews as $review) {
                    $somme_rate += $review['rate'];
                }
                $stats_product['rate'] = $somme_rate / $num_reviews;
                $stats_product['nb_reviews'] = $num_reviews;
            }

            if (!isset($stats_product['nb_reviews']) || 0 == $stats_product['nb_reviews']) {
                return;
            }
            $average_rate_percent = [];
            $average_rate_percent['floor'] = floor($stats_product['rate']) - 1;
            $average_rate_percent['decimals'] = ($stats_product['rate'] - floor($stats_product['rate'])) * 20;

            $use_star_format_image = Configuration::get('AV_FORMAT_IMAGE', null, null, $this->current_shop_id);
            $customized_star_color = (Configuration::get(
                'AV_STARCOLOR',
                null,
                null,
                $this->current_shop_id
            )) ? Configuration::get('AV_STARCOLOR', null, null, $this->current_shop_id) : 'FFCD00';
            // default #FFCD00
            $this->smartyAssign([
                'av_nb_reviews' => $stats_product['nb_reviews'],
                'av_rate' => $stats_product['rate'],
                'average_rate_percent' => $average_rate_percent,
                'av_rate_percent_int' => ($stats_product['rate']) ? round($stats_product['rate'] * 20) : 100,
                'link_product' => $link ? $link : '',
                'use_star_format_image' => $use_star_format_image,
                'average_rate' => round($stats_product['rate'], 1),
                'customized_star_color' => $customized_star_color,
            ]);
            $tpl = 'avisverifies-categorystars';

            return $this->displayTemplate($tpl);
        }
    }

    public function hookDisplayProductListReviews($params)
    {
        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            return $this->hookCategorystarsNetreviews($params);
        }
    }

    public function hookDisplayProductPriceBlock($params)
    {
        if (version_compare(_PS_VERSION_, '1.7', '>')) {
            if (isset($params['type'])) {
                if ('before_price' == $params['type']) {
                    return $this->hookCategorystarsNetreviews($params);
                }
            }
        }
    }

    /**
     * Allow to display the fixed widget.
     */
    public function hookDisplayFooter($params)
    {
        if ('checked' === Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id)) {
            if (null !== $this->group_name) {
                $av_scriptfixe_allowed = Configuration::get(
                    'AV_SCRIPTFIXE_ALLOWED' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
                $av_scriptfixe_position = Configuration::get(
                    'AV_SCRIPTFIXE_POSITION' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
                $av_scriptfixe = Configuration::get(
                    'AV_SCRIPTFIXE' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
            } else {
                $av_scriptfixe_allowed = 'no';
            }
        } else {
            $av_scriptfixe_allowed = Configuration::get(
                'AV_SCRIPTFIXE_ALLOWED',
                null,
                null,
                $this->current_shop_id
            );
            $av_scriptfixe_position = Configuration::get(
                'AV_SCRIPTFIXE_POSITION',
                null,
                null,
                $this->current_shop_id
            );
            $av_scriptfixe = Configuration::get(
                'AV_SCRIPTFIXE',
                null,
                null,
                $this->current_shop_id
            );
        }

        $widget_fix_av = '';
        if ('yes' === $av_scriptfixe_allowed) {
            if ((true != strpos(Tools::strtolower($av_scriptfixe), 'null') || Tools::strlen($av_scriptfixe) > 10) &&
                'yes' === $av_scriptfixe_allowed && 'footer' === $av_scriptfixe_position) {
                $widget_fix_av = "\n\n<div id='wigetfix_avisverifies_footer'>" . Tools::stripslashes(
                    html_entity_decode($av_scriptfixe)
                ) .
                    '</div>';
            }
        }
        // ************************************ Display TagJS *****************************************************************************************
        $scripts = $widget_fix_av;
        if ($this->storeKeysConfigured()) {   // Test if Store / Storefront (multilange) has idWebsite & secretKey
            if (Configuration::get('AV_RELOADED' . $this->group_name, null, null, $this->current_shop_id)) {
                // TagJS script only in display pages, home, product, category
                if (property_exists($this->context->controller, 'php_self') && in_array($this->context->controller->php_self, ['index', 'product', 'category'])) {
                    $scripts .= $this->getTagJSScript();
                }
            }
        }
        // ***************************** End TagJS *****************************************************************************************************
        return $scripts;
    }

    /**
     * allow to display a overview of a category,
     * {hook h='CategorysummaryNetreviews'} to be added in category.tpl.
     */
    public function hookCategorysummaryNetreviews($params)
    {
        // $av_idshop = $this->current_shop_id;
        // // $lang_id = (empty($this->id_lang))?1:$this->id_lang;
        // $current_page_name = $this->context->controller->php_self;
        // if (($current_page_name == 'manufacturer' || $current_page_name == 'category' )) {
        //     // find the list of the id in a manufacturer
        //     if ($current_page_name == 'manufacturer') {
        //         $id_manufacturer=(int)Tools::getValue('id_manufacturer');
        //         $manu = new Manufacturer($id_manufacturer, $this->context->language->id);
        //         $name_subject=$manu->name;
        //         $description_subject = $manu->description;

        //         $sql = 'SELECT id_product FROM '._DB_PREFIX_.'product where id_manufacturer="'.$id_manufacturer.'"';
        //     } elseif ($current_page_name == 'category') {
        //         $id_category=(int)Tools::getValue('id_category');
        //         $cat = new Category($id_category, $this->context->language->id);
        //         $name_subject=$cat->name;
        //         $description_subject = $cat->description;
        //         $sql = 'SELECT * FROM '._DB_PREFIX_.'category_product where id_category="'.$id_category.'"';
        //     }
        //     $results = Db::getInstance()->ExecuteS($sql);
        //     // predefine the stats of the reviews, contains the number and the total of the rates
        //     $stats_product = array('nb_reviews'=>0,'somme'=>0);
        //     $price_sum = 0;
        //     $products_info = array();
        //     foreach ($results as $row) {
        //         $id_product=(int)$row['id_product'];
        //         // $products_info[$id_product]['name'] = $this->getProductName($id_product, $lang_id);
        //         $products_info[$id_product]['price'] = round(Product::getPriceStatic($id_product), 2);
        //         $price_sum += $products_info[$id_product]['price'];
        //         $o_av = new NetReviewsModel();

        //         $reviews = $o_av->getProductReviews($id_product, $this->group_name, $av_idshop, false, 0);

        //         foreach ($reviews as $review) {
        //             // calculate the number of review and the total of the rates
        //             $stats_product['nb_reviews']++;
        //             $stats_product['somme'] = $stats_product['somme'] + $review['rate'];
        //         }
        //     }

        //     if (isset($review)) {
        //         $num_products = (count($results) > 0)? count($results):1;
        //         $price_average = $price_sum / $num_products;
        //         $average_rate_percent = array();
        //         $average_rate_percent['floor'] = floor($review['rate']) - 1;
        //         $average_rate_percent['decimals'] = ($review['rate'] - floor($review['rate']))*20;
        //         $use_star_format_image = Configuration::get('AV_FORMAT_IMAGE', null, null, $av_idshop);
        //         if (version_compare(_PS_VERSION_, '1.4', '>=') && $use_star_format_image != '1') {
        //             $stars_file = 'avisverifies-stars-font.tpl';
        //         } else {
        //             $stars_file = 'avisverifies-stars-image.tpl';
        //             $use_image = true;
        //         }
        //         $brand = Configuration::get('PS_SHOP_NAME');
        //         // calcul de la moyen
        //         if ($stats_product['nb_reviews'] > 0) {
        //             $stats_product['rate'] = $stats_product['somme'] / $stats_product['nb_reviews'];
        //             $this->smartyAssign(array(
        //                 'modules_dir' => _MODULE_DIR_,
        //                 'count_reviews' => $stats_product['nb_reviews'],
        //                 'average_rate' => round($stats_product['rate'], 1),
        //                 'average_rate_percent' => $average_rate_percent,
        //                 'nom_category'=>strip_tags($name_subject),
        //                 'description_category'=> !empty($description_subject)?
        // strip_tags($description_subject): false,
        //                 'page_name'=>$current_page_name,
        //                 'price_average'=>$price_average,
        //                 'stars_dir' => _PS_ROOT_DIR_.'/modules/netreviews/views/templates/hook/sub/'.$stars_file,
        //                 'logo_lang' =>  $this->iso_lang, //$this->context->language->iso_code,
        //                 'brand' =>  $brand, //$this->context->language->iso_code,
        //                 'products_av'=> $products_info
        //             ));
        //             $tpl = 'avisverifies-category-summary';
        //             return $this->displayTemplate($tpl);
        //         }
        //     }
        // }
    }

    /**
     * allow to display a overview of the website,
     * {hook h='GlobalnoteNetreviews'} to be added in header.tpl.
     */
    public function hookGlobalnoteNetreviews($params)
    {
        $rate_site = Configuration::get('AV_RATE_SITE', null, null, $this->current_shop_id);
        $nb_site = Configuration::get('AV_AVIS_SITE', null, null, $this->current_shop_id);
        $horodate = Configuration::get('AV_HORODATE_LASTGET', null, null, $this->current_shop_id);
        $av_idwebsite = Configuration::get('AV_IDWEBSITE' . $this->group_name, null, null, $this->current_shop_id);
        $shop_name = Configuration::get('PS_SHOP_NAME');
        $use_image = false;
        $ex_datas = [];
        if (empty($rate_site) or empty($nb_site) or empty($horodate) or (($horodate + 86400) < time())) {
            $nb_site = null;
            $rate_site = null;
            $url =
                'https://cl.avis-verifies.com/' . $this->iso_lang . '/cache/' . Tools::substr(
                    $av_idwebsite,
                    0,
                    1
                ) . '/' . Tools::substr($av_idwebsite, 1, 1) . '/' . Tools::substr(
                    $av_idwebsite,
                    2,
                    1
                ) . '/' . $av_idwebsite . '/AWS/' . $av_idwebsite . '_infosite.txt';
            $file_headers = @get_headers($url);
            if (strpos($file_headers[0], '200')) {
                $datas = NetReviewsModel::avFileGetContents($url);
                $ex_datas = explode(';', $datas);

                if (1 == Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
                    Configuration::updateValue('AV_HORODATE_LASTGET', time(), false, null, $this->current_shop_id);
                    Configuration::updateValue('AV_AVIS_SITE', $ex_datas[0], false, null, $this->current_shop_id);
                    Configuration::updateValue('AV_RATE_SITE', $ex_datas[1], false, null, $this->current_shop_id);
                } else {
                    Configuration::updateValue('AV_HORODATE_LASTGET', time());
                    Configuration::updateValue('AV_AVIS_SITE', $ex_datas[0]);
                    Configuration::updateValue('AV_RATE_SITE', $ex_datas[1]);
                }
                $nb_site = $ex_datas[0];
                $rate_site = $ex_datas[1];
            }
        } else {
            $use_star_format_image = Configuration::get('AV_FORMAT_IMAGE', null, null, $this->current_shop_id);
            if (version_compare(_PS_VERSION_, '1.4', '>=') && '1' != $use_star_format_image) {
                $stars_file = 'avisverifies-stars-font.tpl';
            } else {
                $stars_file = 'avisverifies-stars-image.tpl';
                $use_image = true;
            }
            $average_rate_percent = [];
            $average_rate_percent['floor'] = floor($rate_site) - 1;
            $average_rate_percent['decimals'] = ($rate_site - floor($rate_site)) * 20;
            $this->smartyAssign([
                'av_site_rating_avis' => $nb_site,
                'stars_dir' => _PS_ROOT_DIR_ . '/modules/netreviews/views/templates/hook/sub/' . $stars_file,
                'use_image' => $use_image,
                'av_site_rating_rate' => round($rate_site, 1),
                'shop_name' => $shop_name,
                'average_rate_percent' => $average_rate_percent,
                'modules_dir' => _MODULE_DIR_,
            ]);
            $tpl = 'header_av_site';

            return $this->displayTemplate($tpl);
        }
    }

    public function hookActionValidateOrder($params)
    {
        // $process_init = Configuration::get('AV_PROCESSINIT');
        $id_website = configuration::get(
            'AV_IDWEBSITE' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $secret_key = configuration::get(
            'AV_CLESECRETE' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $code_lang = configuration::get(
            'AV_CODE_LANG' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $code_lang = (!empty($code_lang)) ? $code_lang : 'undef';

        if (empty($id_website) || empty($secret_key)) {
            return;
        }
        $o_av = new NetReviewsModel();
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $id_order = Tools::getValue('id_order');
        // $order = new Order((int)($id_order));
        // $order_total = (100 * $order->total_paid);
        } else {
            $o_order = $params['order'];
            $id_order = $o_order->id;
            // $order_total = ($o_order->total_paid) ? (100 * $o_order->total_paid) : 0;
            if (!empty($o_order->id_shop)) {
                $o_av->id_shop = $o_order->id_shop;
            }
        }
        if (isset($id_order) && !empty($id_order)) {
            $o_av->id_order = (int) $id_order;
            $o_av->iso_lang = pSQL(Language::getIsoById($this->id_lang));
            // pSQL(Language::getIsoById($o_order->id_lang));
            $o_av->saveOrderToRequest();

            return '';
        }
    }

    public function addLog($message, $severity = 2)
    {
        $now = new DateTime();
        $now->setTimezone(new DateTimeZone('Europe/Paris'));
        $dateTime = $now->format('Y-m-d H:i:s');
        $fileName = dirname(__FILE__) . '/logs.txt';
        try {
            $fp = fopen($fileName, 'a+'); // path: prestashop\modules\netreviews
            if ($fp) {
                $filePerms = $this->getFilePerms($fileName);
                if ($filePerms >= 640 && 700 != $filePerms) {
                    chmod($fileName, 0600);
                    clearstatcache();
                }
                $newFilePerms = $this->getFilePerms($fileName);
                $allowedPerms = [600, 700];
                if (in_array($newFilePerms, $allowedPerms)) {
                    fwrite($fp, $dateTime . ' [Netreviews - ' . $severity . '] ' . Tools::safeOutput($message) . PHP_EOL . PHP_EOL);
                }
            }
            fclose($fp);
        } catch (Exception $e) {
            // no log
            return;
        }
    }

    private function getFilePerms($filename)
    {
        return (int) substr(sprintf('%o', fileperms($filename)), -4);
    }

    /**
     * This code is added for having possiblities of double check
     * if not all orders are registered.
     */
    public function hookActionOrderStatusPostUpdate($params)
    {
        // NETREVIEWS PURCHASE EVENTS SECTION
        $av_reloaded = Configuration::get('AV_RELOADED' . $this->group_name, null, null, $this->current_shop_id);
        $av_activate_collect = Configuration::get('AV_ACTIVATE_COLLECT' . $this->group_name, null, null, $this->current_shop_id);
        if (1 == $av_reloaded && true == $av_activate_collect) {
            $statusList = Configuration::get('AV_ORDERSTATESCHOOSEN' . $this->group_name, null, null, $this->current_shop_id);
            $statusListArray = explode(';', $statusList);

            foreach ($statusListArray as $statusId) {
                if ($params['newOrderStatus']->id == $statusId) {
                    $this->sendOrders();
                }
            }
        }

        $id_website = configuration::get('AV_IDWEBSITE' . $this->group_name, null, null, $this->current_shop_id);
        $secret_key = configuration::get('AV_CLESECRETE' . $this->group_name, null, null, $this->current_shop_id);
        $code_lang = configuration::get('AV_CODE_LANG' . $this->group_name, null, null, $this->current_shop_id);
        $code_lang = (!empty($code_lang)) ? $code_lang : 'undef';
        if (empty($id_website) || empty($secret_key)) {
            return;
        }
        $o_av = new NetReviewsModel();
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            $id_order = Tools::getValue('id_order');
        // $order = new Order((int)($id_order));
        } else {
            $id_order = (int) $params['id_order'];
            if (isset($params['cart']->id_shop) && !empty($params['cart']->id_shop)) {
                $o_av->id_shop = $params['cart']->id_shop;
            }
        }
        if (isset($id_order) && !empty($id_order)) {
            $o_av->id_order = (int) $id_order;
            $o_av->iso_lang = pSQL(Language::getIsoById($this->id_lang));
            // pSQL(Language::getIsoById($o_order->id_lang));
            $o_av->saveOrderToRequest();

            return '';
        }
    }

    public function getLastIdOrder($idShop, $idCustomer)
    {
        if (null == $idShop && _PS_VERSION_ < '1.7' && _PS_VERSION_ >= '1.6') {
            $idShop = 1;
        }
        $query = 'SELECT id_order FROM ' . _DB_PREFIX_ . 'orders WHERE id_shop = ' . $idShop . ' 
        and id_customer = ' . $idCustomer . ' ORDER BY date_add DESC';
        $orderId = Db::getInstance()->getRow($query, false);

        return $orderId;
    }

    public function hookDisplayPaymentTop($params)
    {
        if ('checked' == Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id)) {
            if (Configuration::hasKey('AV_COLLECT_CONSENT', null, null, $this->current_shop_id)) {
                Configuration::deleteByName('AV_COLLECT_CONSENT');
            }
            if (Configuration::hasKey('AV_CONSENT_ANSWER_NO', null, null, $this->current_shop_id)) {
                Configuration::deleteByName('AV_CONSENT_ANSWER_NO');
            }
            $key = 'AV_COLLECT_CONSENT' . $this->group_name;
        } else {
            $key = 'AV_COLLECT_CONSENT';
        }

        $idCustomer = $this->context->cart->id_customer;

        if (Configuration::hasKey($key, null, null, $this->current_shop_id)) {
            $collectConsent = Configuration::get($key, null, null, $this->current_shop_id);
            $url = $this->context->link->getModuleLink('netreviews', 'ajax');
            if ('yes' === $collectConsent) {
                $tpl = 'avisverifies-checkbox-consent';
                $this->smartyAssign([
                    'url' => $url,
                    'idShop' => $this->current_shop_id,
                    'groupName' => $this->group_name,
                    'idCustomer' => $idCustomer,
                    'prestashopVersion' => _PS_VERSION_,
                ]);

                return $this->displayTemplate($tpl);
            }
        }
    }

    public function hookDisplayRightColumn($params)
    {
        if ('checked' === Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id)) {
            if (null !== $this->group_name) {
                $av_scriptfixe_allowed = Configuration::get(
                    'AV_SCRIPTFIXE_ALLOWED' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
                $av_scriptfixe_position = Configuration::get(
                    'AV_SCRIPTFIXE_POSITION' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
                $av_scriptfixe = Configuration::get(
                    'AV_SCRIPTFIXE' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
            } else {
                $av_scriptfixe_allowed = 'no';
            }
        } else {
            $av_scriptfixe_allowed = Configuration::get(
                'AV_SCRIPTFIXE_ALLOWED',
                null,
                null,
                $this->current_shop_id
            );
            $av_scriptfixe_position = Configuration::get(
                'AV_SCRIPTFIXE_POSITION',
                null,
                null,
                $this->current_shop_id
            );
            $av_scriptfixe = Configuration::get(
                'AV_SCRIPTFIXE',
                null,
                null,
                $this->current_shop_id
            );
        }
        if ('yes' != $av_scriptfixe_allowed || 'right' != $av_scriptfixe_position) {
            return;
        }
        if ((true != strpos(Tools::strtolower($av_scriptfixe), 'null') || Tools::strlen($av_scriptfixe) > 10) &&
            'yes' === $av_scriptfixe_allowed) {
            return "\n\n<div id='wigetfix_avisverifies_rightcolumn'>" . Tools::stripslashes(
                html_entity_decode($av_scriptfixe)
            ) .
                '</div>';
        }
    }

    public function hookProductTab($params)
    {
        $id_product = (int) Tools::getValue('id_product');
        $display_prod_reviews = Configuration::get(
            'AV_DISPLAYPRODREVIEWS' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $avisverifies_rename_tag = Configuration::get('AV_TABNEWNAME', null, null, $this->current_shop_id);
        // ************************************ Display TagJS *****************************************************************************************
        if ($this->storeKeysConfigured()) {   // Test if Store / Storefront (multilange) has idWebsite & secretKey
            if (Configuration::get('AV_RELOADED' . $this->group_name, null, null, $this->current_shop_id)) {
                if ($this->checkProductHasReview($id_product)) {
                    $reloaded_title = (isset($avisverifies_rename_tag) && !empty($avisverifies_rename_tag)) ? $avisverifies_rename_tag : $this->l('verified reviews');
                    $this->smartyAssign(
                        [
                            'count_reviews' => '',
                            'avisverifies_rename_tag' => '',
                            'reloaded' => true,
                            'reloaded_title' => $reloaded_title,
                        ]
                    );

                    $tpl = 'avisverifies-tab';

                    return $this->displayTemplate($tpl);
                }
            }
        }
        // ***************************** End TagJS *****************************************************************************************************
        $o_av = new NetReviewsModel();

        if (!$this->all_multishops_reviews && !$this->all_languages_reviews) {
            $reviews = $o_av->getStatsProduct($id_product, $this->group_name, $this->current_shop_id);
            $num_reviews = !empty($reviews['nb_reviews']) ? $reviews['nb_reviews'] : 0;

            $show_tag = Configuration::get('AV_TABSHOW', null, null, $this->current_shop_id); // 1 or null in defaut
            if ($num_reviews < 1 || 'yes' != $display_prod_reviews || '0' === $show_tag) {
                return; // Si Aucun avis, on retourne vide
            }
        } else {
            // if (!$this->all_languages_reviews && !$this->group_name) {
            //     return;
            // }
            $language_group = ($this->all_languages_reviews) ? null : $this->group_name;
            $shop_id_group = ($this->all_multishops_reviews) ? null : $this->current_shop_id;
            $reviews = $o_av->getProductReviews(
                $id_product,
                $language_group,
                $shop_id_group,
                0,
                1,
                'horodate_DESC',
                0,
                false
            );
            $num_reviews = count($reviews);
            if ($num_reviews < 1 || 'yes' != $display_prod_reviews) {
                return;
            }
            $somme_rate = 0;
            foreach ($reviews as $review) {
                $somme_rate += $review['rate'];
            }
            $reviews['rate'] = $somme_rate / $num_reviews;
            $reviews['nb_reviews'] = $num_reviews;
        }
        $this->smartyAssign(
            [
                'count_reviews' => $reviews['nb_reviews'],
                'avisverifies_rename_tag' => (Tools::strlen(trim(
                    $avisverifies_rename_tag
                )) > 3) ? $avisverifies_rename_tag : '',
                'reloaded' => false,
                'reloaded_title' => '',
            ]
        );

        $tpl = 'avisverifies-tab';

        return $this->displayTemplate($tpl);
    }

    /**
     * Integration of TagJS
     * List of Reviews.
     */
    public function displayReviewTagJS($product, $lang_id, $new_version)
    {
        // Configuration Variables
        $display_prod_reviews = configuration::get('AV_DISPLAYPRODREVIEWS' . $this->group_name, null, null, $this->current_shop_id);
        $av_reviewstagjs = configuration::get('AV_REVIEWSTAGJS' . $this->group_name, null, null, $this->current_shop_id);
        $av_script = configuration::get('AV_SCRIPTTAGJS' . $this->group_name, null, null, $this->current_shop_id);
        $av_reviewstagjs = Tools::stripslashes(html_entity_decode($av_reviewstagjs));
        $display_prod_reviews = ('yes' == $display_prod_reviews);

        // Rich Snippets attributes data-product-img-url, data-product-url, data-product-name, data-product-price, data-product-id, data-product-rs-id
        $url_page = NetReviewsModel::getUrlProduct($product->id, $lang_id);
        $product_url = !empty($url_page) ? $url_page : false;
        $url_image = NetReviewsModel::getUrlImageProduct($product->id, null, $lang_id);
        $url_image = !empty($url_image) ? $url_image : false;
        $product_rs_id = $product_url;
        $product_price = $product->getPrice(true, null, 2);
        $product_price = !empty($product_price) ? $product_price : 0;
        $id_product = $product->id;
        if (is_array($product->name)) {
            if (isset($product->name[$lang_id])) {
                $product_name = $product->name[$lang_id];
            } else {
                $product_name = array_values($product->name)[0];
            }
        } else {
            $product_name = $product->name;
        }
        $product_name = !empty($product_name) ? $product_name : 'product name';

        // Please note TagJS requires following variables, AverageTagJS, ScriptTagJS, AV_DISPLAYPRODREVIEWS
        if (!empty($av_reviewstagjs) && !empty($av_script) && $display_prod_reviews) {
            $this->context->smarty->assign('product_id', $id_product);
            $this->context->smarty->assign('product_price', $product_price);
            $this->context->smarty->assign('product_name', $product_name);
            $this->context->smarty->assign('product_url', $product_url);
            $this->context->smarty->assign('url_image', $url_image);
            try {
                // https://www.smarty.net/forums/viewtopic.php?p=94650&sid=81185d158408ef1d636d6a7dcc20c55c
                $content = $this->context->smarty->fetch('string: ' . $av_reviewstagjs);
                if ($new_version) {
                    // ProductExtraContent variables
                    $attribute_av = ['id' => 'netreviews_tab', 'class' => 'netreviews_tab'];
                    $title_new = Configuration::get('AV_TABNEWNAME', null, null, $this->current_shop_id);
                    $title_string = $this->l('verified reviews');
                    $title = (isset($title_new) && !empty($title_new)) ? $title_new : $title_string;

                    return $this->getExtraContent($attribute_av, $title, $content);
                } else {
                    return $content;
                }
            } catch (Exception $e) {
                return;
            }
        } else {
            // Then store / storefront is active for TagJS but has either chosen no to display reviews or is missing configuration variables (Tag or Script or both)
            return;
        }
    }

    /**
     * Utilised in hookTabcontentNetreviews.
     */
    public function getExtraContent($attribute_av, $title, $content)
    {
        $extraContent = (new PrestaShop\PrestaShop\Core\Product\ProductExtraContent()); // need to be disabled if have T_string, class missing errors
        $extraContent->setAttr($attribute_av);
        $extraContent->setTitle($title);
        $extraContent->setContent($content);
        $array[] = $extraContent;

        return $array;
    }

    /**
     * Modifications in $reviews need to be duplicated in ajax-load.php
     * Display reviews on the product page, used 3 hooks
     * hookTabcontentNetreviews (parent hook).
     *
     * hookDisplayProductExtraContent (version > 1.7)
     * hookProductTabContent (all versions)
     */
    public function hookTabcontentNetreviews($params)
    {
        // if it's not product page, the reviews won't display
        if (property_exists($this->context->controller, 'php_self') &&
            'product' != $this->context->controller->php_self) {
            return;
        }

        if (!isset($params['product']->use_tabconent[0])) {
            $new_version = false;
        } else {
            $use_tab_content = $params['product']->use_tabconent[0];
            $new_version = (!$use_tab_content) ? true : false;
        }

        $display_prod_reviews = configuration::get(
            'AV_DISPLAYPRODREVIEWS' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $url_certificat = Configuration::get(
            'AV_URLCERTIFICAT' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $avisverifies_nb_reviews = (int) Configuration::get(
            'AV_NBOFREVIEWS',
            null,
            null,
            $this->current_shop_id
        );
        $avisverifies_nb_reviews = (!empty($avisverifies_nb_reviews)) ? $avisverifies_nb_reviews : 5;
        // in case that AV_NBOFREVIEWS is not defined after an update

        $local_id_website = Configuration::get(
            'AV_IDWEBSITE' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $local_secure_key = Configuration::get(
            'AV_CLESECRETE' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $shop_name = Configuration::get('PS_SHOP_NAME');
        $id_product = (int) Tools::getValue('id_product');
        $o_av = new NetReviewsModel();

        $stats_product = (!isset($this->stats_product) || empty($this->stats_product)) ?
            $o_av->getStatsProduct($id_product, $this->group_name, $this->current_shop_id) : $this->stats_product;
        // ************************************ Display TagJS *****************************************************************************************
        $lang_id = (isset($this->context->language->id) && !empty($this->context->language->id)) ? (int) $this->context->language->id : 1;
        if (version_compare(_PS_VERSION_, '1.5', '>=')) {
            $product = new Product((int) $id_product, false, $lang_id);
        } else {
            $product = new Product((int) $id_product);
        }
        if ($this->storeKeysConfigured()) {   // Test if Store / Storefront (multilange) has idWebsite & secretKey
            if (Configuration::get('AV_RELOADED' . $this->group_name, null, null, $this->current_shop_id)) {
                if ($this->checkProductHasReview($id_product)) {
                    return $this->displayReviewTagJS($product, $lang_id, $new_version);
                }
            }
        }
        // ***************************** End TagJS *****************************************************************************************************
        if ($this->all_multishops_reviews || $this->all_languages_reviews) { // override rate & nb_reviews
            $language_group = ($this->all_languages_reviews) ? null : $this->group_name;
            $shop_id_group = ($this->all_multishops_reviews) ? null : $this->current_shop_id;
            $reviews = $o_av->getProductReviews(
                $id_product,
                $language_group,
                $shop_id_group,
                0,
                1,
                'horodate_DESC',
                0,
                false
            );
            $num_reviews = count($reviews);
            if ($num_reviews < 1 || 'yes' != $display_prod_reviews) {
                return;
            }
            $somme_rate = 0;
            foreach ($reviews as $review) {
                $somme_rate += $review['rate'];
            }
            $stats_product['rate'] = $somme_rate / $num_reviews;
            $stats_product['nb_reviews'] = $num_reviews;
        } else {
            $language_group = $this->group_name;
            $shop_id_group = $this->current_shop_id;
        }

        if (!empty($stats_product['nb_reviews']) && 'yes' == $display_prod_reviews) {
            $reviews_list = []; // Create array with all reviews data
            $my_review = []; // Create array with each reviews data

            $reviews = $o_av->getProductReviews(
                $id_product,
                $language_group,
                $shop_id_group,
                $avisverifies_nb_reviews,
                1,
                'horodate_DESC',
                0,
                false
            );
            $reviews_count = $stats_product['nb_reviews'];

            if ($avisverifies_nb_reviews < 1 || empty($avisverifies_nb_reviews)) {
                $avisverifies_nb_reviews = 10;
            }

            $reviews_max_pages = floor($reviews_count / $avisverifies_nb_reviews) +
                ($reviews_count % $avisverifies_nb_reviews > 0 ? 1 : 0);

            $reviews_rate_portion = [];
            $reviews_rate_portion_persontage = [];
            $reviews_rate_portion_keys = [1, 2, 3, 4, 5];
            $reviews_rate_portion = array_fill_keys($reviews_rate_portion_keys, 0);

            $reviews_all = $o_av->getProductReviews(
                $id_product,
                $language_group,
                $shop_id_group,
                0,
                1,
                'horodate_DESC',
                0,
                false
            );

            foreach ($reviews_all as $review) {
                switch ($review['rate']) {
                    case '1':
                        ++$reviews_rate_portion[1];
                        break;
                    case '2':
                        ++$reviews_rate_portion[2];
                        break;
                    case '3':
                        ++$reviews_rate_portion[3];
                        break;
                    case '4':
                        ++$reviews_rate_portion[4];
                        break;
                    case '5':
                        ++$reviews_rate_portion[5];
                        break;
                }
            }

            foreach ($reviews_rate_portion as $r_rate => $r_num) {
                $reviews_rate_portion_persontage[$r_rate] = round(($r_num / $stats_product['nb_reviews']) * 100, 0);
            }
            $reviewsHigherThan3 = $reviews_rate_portion[5] + $reviews_rate_portion[4] + $reviews_rate_portion[3];
            $percentageRecommendingProduct = round(($reviewsHigherThan3 / $stats_product['nb_reviews']) * 100, 0);

            foreach ($reviews as $review) {
                // Create variable for template engine
                $my_review['ref_produit'] = $review['ref_product'];
                $my_review['id_product_av'] = $review['id_product_av'];
                $my_review['sign'] = sha1($local_id_website . $review['id_product_av'] . $local_secure_key);
                if (!isset($review['helpful']) && !isset($review['helpless'])) {
                    Db::getInstance()->Execute('ALTER TABLE `' . _DB_PREFIX_ . 'av_products_reviews`
                   ADD `helpful` int(7) DEFAULT 0,
                   ADD `helpless` int(7) DEFAULT 0');
                } else {
                    $my_review['helpful'] = $review['helpful'];
                    $my_review['helpless'] = $review['helpless'];
                }
                $my_review['rate'] = $review['rate'];
                $my_review['rate_percent'] = $review['rate'] * 20;
                $my_review['avis'] = html_entity_decode(urldecode($review['review']));
                // review date
                if ('10' == Tools::strlen($review['horodate'])) {
                    $date = new DateTime();
                    $date->setTimestamp($review['horodate']);
                    $my_review['horodate'] = $date->format('d/m/Y');
                } else {
                    $my_review['horodate'] = date('d/m/Y', strtotime($review['horodate']));
                }
                // order date
                if (isset($review['horodate_order']) && !empty($review['horodate_order'])) {
                    $review['horodate_order'] = str_replace('"', '', $review['horodate_order']);
                    $my_review['horodate_order'] = date('d/m/Y', strtotime($review['horodate_order']));
                } else {
                    $my_review['horodate_order'] = $my_review['horodate'];
                }
                // in case imported reviews which have lack of this info
                if (!isset($review['horodate']) || empty($review['horodate'])) {
                    $my_review['horodate'] = $my_review['horodate_order'];
                }

                $my_review['customer_name'] = urldecode($review['customer_name']);

                $my_review['discussion'] = [];

                // renverser le nom et le prénom
                $customer_name = explode(' ', urldecode($review['customer_name']));
                $customer_name = array_values(array_filter($customer_name));
                $customer_name = array_diff($customer_name, ['.']);
                $my_review['customer_name_anonymous'] = ('Anonymous' == $customer_name[1]) ? true : false;
                $customer_name = array_reverse($customer_name);
                $customer_name = implode(' ', $customer_name);

                $my_review['customer_name'] = $customer_name;

                $json_discussion = NetReviewsModel::avJsonDecode(NetReviewsModel::acDecodeBase64($review['discussion']), true);
                $json_discussion = (array) $json_discussion;
                if ($json_discussion) {
                    foreach ($json_discussion as $k_discussion => $each_discussion) {
                        $each_discussion = (array) $each_discussion;
                        $my_review['discussion'][$k_discussion] = [];
                        if ('10' == Tools::strlen($each_discussion['horodate'])) {
                            $date = new DateTime();
                            $date->setTimestamp($each_discussion['horodate']);
                            $my_review['discussion'][$k_discussion]['horodate'] = $date->format('d/m/Y');
                        } else {
                            $my_review['discussion'][$k_discussion]['horodate'] =
                                date('d/m/Y', strtotime($each_discussion['horodate']));
                        }
                        $my_review['discussion'][$k_discussion]['commentaire'] =
                        urldecode($each_discussion['commentaire']);
                        if ('ecommercant' == $each_discussion['origine']) {
                            $my_review['discussion'][$k_discussion]['origine'] = $shop_name;
                        } elseif ('internaute' == $each_discussion['origine']) {
                            $my_review['discussion'][$k_discussion]['origine'] = $my_review['customer_name'];
                        } else {
                            $my_review['discussion'][$k_discussion]['origine'] = $this->l('Moderator');
                        }
                    }
                }
                // Media infos
                $my_review['media_content'] = [];
                if (isset($review['media_full'])) {
                    $review_images_result = (array) NetReviewsModel::avJsonDecode(
                        html_entity_decode(
                            $review['media_full']
                        ),
                        true
                    );
                    if (isset($review_images_result) && !empty($review_images_result)
                        && count($review_images_result) >= 1) {
                        foreach ($review_images_result as $k_media => $each_media) {
                            $my_review['media_content'][$k_media] = (array) $each_media;
                        }
                    }
                }

                array_push($reviews_list, $my_review);
            }

            $customized_star_color = (Configuration::get(
                'AV_STARCOLOR',
                null,
                null,
                $this->current_shop_id
            )) ? Configuration::get(
                'AV_STARCOLOR',
                null,
                null,
                $this->current_shop_id
            ) : 'FFCD00';
            // default #FFCD00
            $nrResponsive = Configuration::get('AV_NRESPONSIVE', null, null, $this->current_shop_id) ? 1 : 0;
            // 0 or null in defaut
            $hidehelpful = Configuration::get('AV_HELPFULHIDE', null, null, $this->current_shop_id) ? 1 : 0;
            // 0 or null in defaut
            $hidemedia = Configuration::get('AV_MEDIAHIDE', null, null, $this->current_shop_id);
            // 0 or null in defaut

            $av_urlcertificat = Configuration::get(
                'AV_URLCERTIFICAT' . $this->group_name,
                null,
                null,
                $this->current_shop_id
            );
            $url_platform = explode('/', $av_urlcertificat);
            $platform = Tools::substr($url_platform[2], 4);
            $avHelpfulURL = 'https://www.' . $platform . '/index.php?action=act_api_product_reviews_helpful';
            $url_cgv = 'https://www.' . $platform . '/index.php?page=mod_conditions_utilisation';
            $average_rate_percent = [];
            $average_rate_percent['floor'] = floor($stats_product['rate']) - 1;
            $average_rate_percent['decimals'] = ($stats_product['rate'] - floor($stats_product['rate'])) * 20;
            if (version_compare(_PS_VERSION_, '1.5', '<')) {
                global $cookie;
                $logolang = pSQL(Language::getIsoById($cookie->id_lang));
            } else {
                $logolang = $this->context->language->iso_code;
            }
            $languagespack_av = ['de', 'en', 'es', 'fr', 'gb', 'it', 'pt'];
            $logolang = (in_array($logolang, $languagespack_av)) ? $logolang : 'en';

            $use_star_format_image = Configuration::get('AV_FORMAT_IMAGE', null, null, $this->current_shop_id);
            $use_image = false;
            if (version_compare(_PS_VERSION_, '1.4', '>=') && '1' != $use_star_format_image) {
                $stars_file = 'avisverifies-stars-font.tpl';
                $old_lang = false;
            } else {
                $stars_file = 'avisverifies-stars-image.tpl';
                $old_lang = true;
                $use_image = true;
            }
            $av_ajax_translation = [];
            $av_ajax_translation['a'] = $this->l('published');
            $av_ajax_translation['b'] = $this->l('the');
            $av_ajax_translation['c'] = $this->l('following an order made on');
            $av_ajax_translation['d'] = $this->l('Comment from');
            $av_ajax_translation['e'] = $this->l('Show exchanges');
            $av_ajax_translation['f'] = $this->l('Hide exchanges');
            $av_ajax_translation['g'] = $this->l('Did you find this helpful?');
            $av_ajax_translation['h'] = $this->l('Yes');
            $av_ajax_translation['i'] = $this->l('No');
            $av_ajax_translation['j'] = $this->l('More reviews...');
            $av_template = Configuration::get('AV_TEMPLATE', null, null, $this->current_shop_id);
            if ($av_template && '2' == $av_template) {
                $ajax_dir = NetReviewsModel::tplFileExist('sub/ajax-load-tab-content-design-new.tpl');
                $design_template = 'avisverifies-tab-content-design-new.tpl';
            } else {
                $ajax_dir = NetReviewsModel::tplFileExist('ajax-load-tab-content.tpl');
                $design_template = 'avisverifies-tab-content-design-classic.tpl';
            }
            $stars_dir = NetReviewsModel::tplFileExist('sub/' . $stars_file);
            $design_dir = NetReviewsModel::tplFileExist('sub/' . $design_template);
            $this->smartyAssign([
                'modules_dir' => _MODULE_DIR_,
                'current_url' => $_SERVER['REQUEST_URI'],
                'av_idwebsite' => $local_id_website,
                'avHelpfulURL' => $avHelpfulURL,
                'url_cgv' => $url_cgv,
                'version_ps' => _PS_VERSION_,
                'ajax_dir' => $ajax_dir,
                'stars_dir' => $stars_dir,
                'design_dir' => $design_dir,
                'use_image' => $use_image,
                'id_shop' => $this->current_shop_id,
                'nom_group' => (!empty($this->group_name)) ? $this->group_name : null,
                'reviews' => $reviews_list,
                'count_reviews' => $reviews_count,
                'average_rate' => round($stats_product['rate'], 1),
                'av_rate_percent_int' => (float) $stats_product['rate'] * 20,
                'average_rate_percent' => $average_rate_percent,
                'reviews_rate_portion_persontage' => $reviews_rate_portion_persontage,
                'is_https' => (array_key_exists('HTTPS', $_SERVER) && 'on' == $_SERVER['HTTPS'] ? 1 : 0),
                'url_certificat' => $url_certificat,
                'reviews_max_pages' => ($reviews_max_pages) ? (int) $reviews_max_pages : '',
                'reviews_rate_portion' => $reviews_rate_portion,
                'nrResponsive' => $nrResponsive,
                'hidehelpful' => $hidehelpful,
                'hidemedia' => $hidemedia,
                'current_page' => 1,
                'av_ajax_translation' => $av_ajax_translation,
                'old_lang' => $old_lang, // old version language variable translations
                'logo_lang' => $logolang,
                'customized_star_color' => $customized_star_color,
                'product_id' => $id_product,
                'percentageRecommendingProduct' => $percentageRecommendingProduct,
            ]);
        }
        $tpl = 'avisverifies-tab-content';
        $attribute_av = ['id' => 'netreviews_tab', 'class' => 'netreviews_tab'];
        // if version >= 1.7
        $array = [];
        if ($new_version) {
            if (!empty($stats_product['nb_reviews']) && 'yes' == $display_prod_reviews) {
                $title_new = Configuration::get('AV_TABNEWNAME', null, null, $this->current_shop_id);
                $title_string = $this->l('verified reviews') . '(' . $stats_product['nb_reviews'] . ')';
                $title = (isset($title_new) && !empty($title_new)) ? $title_new : $title_string;
                $content = $this->displayTemplate($tpl);
            } else {
                $title = '';
                $content = '';
            }

            return $this->getExtraContent($attribute_av, $title, $content);
        } else {
            if (!empty($stats_product['nb_reviews']) && 'yes' == $display_prod_reviews) {
                return $this->displayTemplate($tpl);
            }
        }
    }

    public function checkProductHasReview($idProduct)
    {
        // By default we show the tab - if product api is down, it will not prevent reviews from being displayed
        $productHasReviews = true;

        $paramsCurl = [
            'query' => 'average',
            'idWebsite' => Configuration::get('AV_IDWEBSITE' . $this->group_name, null, null, $this->current_shop_id),
            'product' => $idProduct,
            'plateforme' => Configuration::get('AV_PLATEFORME' . $this->group_name, null, null, $this->current_shop_id),
        ];

        $url = 'https://awsapis3.netreviews.eu/product?' . http_build_query($paramsCurl);

        try {
            $curl = curl_init($url);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_HTTPHEADER, [
                'Content-Type: application/json',
                'Host: awsapis3.netreviews.eu',
            ]);
            curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 0);
            curl_setopt($curl, CURLOPT_TIMEOUT, 30);
            $response = curl_exec($curl);
            $responseCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $curl_errno = curl_errno($curl);
            $curl_error = curl_error($curl);

            if (false === $response) {
                $this->addLog('CURL ProductApi responseCode (' . $responseCode . ') ERROR(' . $curl_errno . '): ' . print_r($curl_error, true), 'CURL Error');
            }

            $responseContent = json_decode($response, true);
            if (empty($responseContent)) {
                $productHasReviews = false;
            }
        } catch (Exception $e) {
            $this->addLog('Error on call to API Product ' . $e->getMessage(), 'CURL Error');
        }

        return $productHasReviews;
    }

    public function hookDisplayProductExtraContent($params)
    {
        $params['product']->use_tabconent[] = false;

        return $this->hookTabcontentNetreviews($params);
    }

    public function hookProductTabContent($params)
    {
        $params['product']->use_tabconent[] = true;

        return $this->hookTabcontentNetreviews($params);
    }

    /**
     * Test if Store / Storefront (multilangue) has idWebsite & SecretKey.
     */
    public function storeKeysConfigured()
    {
        $av_multilingue = ('checked' === Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id));
        $id_website = Configuration::get('AV_IDWEBSITE' . $this->group_name, null, null, $this->current_shop_id);
        $secure_key = Configuration::get('AV_CLESECRETE' . $this->group_name, null, null, $this->current_shop_id);
        // Step 1: Test that the store/storefront(multilangue) - hasKeys aka idWebsite & secretKey
        $hasKeys = false;
        if ($av_multilingue) {
            $hasKeys = !empty($this->group_name);  // $this->group_name will be null if multilangue and no idWebsite & secretKey registered on the storefront
        } else {
            $hasKeys = !empty($id_website) && !empty($secure_key);
        }

        return $hasKeys;
    }

    /**
     * Integration of TagJS
     * Average.
     */
    public function displayAverageTagJS($product)
    {
        $display_prod_reviews = configuration::get('AV_DISPLAYPRODREVIEWS' . $this->group_name, null, null, $this->current_shop_id);
        $av_averagetagjs = configuration::get('AV_AVERAGETAGJS' . $this->group_name, null, null, $this->current_shop_id);
        $av_script = configuration::get('AV_SCRIPTTAGJS' . $this->group_name, null, null, $this->current_shop_id);
        $av_averagetagjs = Tools::stripslashes(html_entity_decode($av_averagetagjs));
        $display_prod_reviews = ('yes' == $display_prod_reviews);
        $id_product = $product->id;

        // Please note TagJS requires following variables, AverageTagJS, ScriptTagJS, AV_DISPLAYPRODREVIEWS
        if (!empty($av_averagetagjs) && !empty($av_script) && $display_prod_reviews) {
            $this->context->smarty->assign('product_id', $id_product);
            try {
                // https://www.smarty.net/forums/viewtopic.php?p=94650&sid=81185d158408ef1d636d6a7dcc20c55c
                $av_averagetagjs = $this->context->smarty->fetch('string: ' . $av_averagetagjs);

                return "\n" . $av_averagetagjs;
            } catch (Exception $e) {
                return;
            }
        } else {
            // Then store / storefront is active for TagJS but has either chosen no to display reviews or is missing configuration variables (Tag or Script or both)
            return;
        }
    }

    /**
     * Integration of widget product
     * hookExtraRight
     * hookExtraLeft
     * hookDisplayProductButtons
     * hookExtraNetreviews.
     *
     * AV_LIGHTWIDGET :
     *  1 : simple stars
     *  2 : widget by defaut
     *  3 : widget badge
     */
    public function hookExtraNetreviews()
    {
        $widgetlight = Configuration::get(
            'AV_LIGHTWIDGET',
            null,
            null,
            $this->current_shop_id
        );

        $av_sp_active = Configuration::get(
            'AV_DISPLAYSNIPPETSITE',
            null,
            null,
            $this->current_shop_id
        );
        if ('1' == $av_sp_active) {
            $snippets_active = true;
        }

        if (isset($snippets_active)) {
            $snippets_type = configuration::get(
                'AV_SNIPPETSITETYPE',
                null,
                null,
                $this->current_shop_id
            );
            $website_snippets = configuration::get(
                'AV_RICHSNIPPETSWEBSITE',
                null,
                null,
                $this->current_shop_id
            );
            if ('1' == $website_snippets) {
                $snippets_complete = '0';
            } else {
                $snippets_complete = '1';
            }
        }

        $display_prod_reviews = configuration::get(
            'AV_DISPLAYPRODREVIEWS' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        );
        $id_product = (int) Tools::getValue('id_product');

        $o_av = new NetReviewsModel();
        $lang_id = (isset($this->context->language->id) && !empty($this->context->language->id)) ?
            (int) $this->context->language->id : 1;

        if (version_compare(_PS_VERSION_, '1.5', '>=')) {
            $product = new Product((int) $id_product, false, $lang_id);
        } else {
            $product = new Product((int) $id_product);
        }
        // ************************************ Display TagJS *****************************************************************************************
        if ($this->storeKeysConfigured()) {   // Test if Store / Storefront (multilange) has idWebsite & secretKey
            if (Configuration::get('AV_RELOADED' . $this->group_name, null, null, $this->current_shop_id)) {
                return $this->displayAverageTagJS($product);
            }
        }
        // ***************************** End TagJS *****************************************************************************************************

        if (!$this->all_multishops_reviews && !$this->all_languages_reviews) {
            $reviews = $o_av->getStatsProduct($id_product, $this->group_name, $this->current_shop_id);
            if (isset($snippets_active)) {
                $detailsReviews = $o_av->getProductReviews($id_product, $this->group_name, $this->current_shop_id);
                if (!empty($detailsReviews)) {
                    foreach ($detailsReviews as $k => $review) {
                        $detailsReviews[$k]['review'] = urldecode($review['review']);
                        $detailsReviews[$k]['customer_name'] = urldecode($review['customer_name']);
                    }
                }
            }

            $num_reviews = !empty($reviews['nb_reviews']) ? $reviews['nb_reviews'] : 0;
            if ($num_reviews < 1 || 'yes' != $display_prod_reviews) {
                return;
            }
        } else {
            $language_group = ($this->all_languages_reviews) ? null : $this->group_name;
            $shop_id_group = ($this->all_multishops_reviews) ? null : $this->current_shop_id;
            $reviews = $o_av->getProductReviews(
                $id_product,
                $language_group,
                $shop_id_group,
                0,
                1,
                'horodate_DESC',
                0,
                false
            );
            $num_reviews = count($reviews);
            if ($num_reviews < 1 || 'yes' != $display_prod_reviews) {
                return;
            }
            $somme_rate = 0;
            foreach ($reviews as $review) {
                $somme_rate += $review['rate'];
            }
            $reviews['rate'] = $somme_rate / $num_reviews;
        }

        $average_rate_percent = [];
        $average_rate_percent['floor'] = floor($reviews['rate']) - 1;
        $average_rate_percent['decimals'] = ($reviews['rate'] - floor($reviews['rate'])) * 20;

        if (version_compare(_PS_VERSION_, '1.5', '>=')) {
            $product_description = strip_tags($product->description_short);
        } else {
            if (null != Tools::getValue('id_lang')) {
                $lang = (int) Tools::getValue('id_lang');
            } else {
                $lang = Configuration::get('PS_LANG_DEFAULT');
            }
            $product_description = strip_tags($product->description_short[$lang]);
        }
        $url_page = NetReviewsModel::getUrlProduct($product->id, $lang_id);
        $url_image = NetReviewsModel::getUrlImageProduct($product->id, null, $lang_id);
        $sku = $product->reference;
        $mpn = $product->supplier_reference;
        $gtin_upc = (isset($product->upc) && !empty($product->upc)) ? $product->upc : '';
        $gtin_ean = $product->ean13;
        $brand_name = '';
        $manufacturer = new Manufacturer($product->id_manufacturer, (int) $this->id_lang);
        if (isset($manufacturer->name)) {
            $brand_name = $manufacturer->name;
        }

        $product_price = $product->getPrice(true, null, 2);
        $currency = $this->context->currency->iso_code;

        if (is_array($product->name)) {
            if (isset($product->name[$lang_id])) {
                $product_name = $product->name[$lang_id];
            } else {
                $product_name = array_values($product->name)[0];
            }
        } else {
            $product_name = $product->name;
        }
        $use_star_format_image = Configuration::get('AV_FORMAT_IMAGE', null, null, $this->current_shop_id);
        if (version_compare(_PS_VERSION_, '1.4', '>=') && '1' != $use_star_format_image) {
            $stars_file = 'avisverifies-stars-font.tpl';
        } else {
            $stars_file = 'avisverifies-stars-image.tpl';
        }

        $customized_star_color = (Configuration::get(
            'AV_STARCOLOR',
            null,
            null,
            $this->current_shop_id
        )) ? Configuration::get(
            'AV_STARCOLOR',
            null,
            null,
            $this->current_shop_id
        ) : 'FFCD00'; // default #FFCD00
        $this->smartyAssign([
            'modules_dir' => _MODULE_DIR_,
            'base_url' => __PS_BASE_URI__,
            'version_ps' => _PS_VERSION_,
            'stars_dir' => _PS_ROOT_DIR_ . '/modules/netreviews/views/templates/hook/sub/' . $stars_file,
            'av_nb_reviews' => $num_reviews,
            'av_rate' => round($reviews['rate'], 1),
            'average_rate_percent' => $average_rate_percent,
            'av_rate_percent_int' => ($reviews['rate']) ? round($reviews['rate'] * 20) : 100,
            'average_rate' => round($reviews['rate'], 1),
            'product_id' => $id_product,
            'product_name' => !empty($product_name) ? $product_name : 'product name',
            'product_description' => !empty($product_description) ? $product_description : false,
            'product_url' => !empty($url_page) ? $url_page : false,
            'url_image' => !empty($url_image) ? $url_image : false,
            'product_price' => !empty($product_price) ? $product_price : 0,
            'product_currency' => !empty($currency) ? $currency : 'EUR',
            'sku' => !empty($sku) ? $sku : false,
            'mpn' => !empty($mpn) ? $mpn : false,
            'gtin_upc' => !empty($gtin_upc) ? $gtin_upc : false,
            'gtin_ean' => !empty($gtin_ean) ? $gtin_ean : false,
            'brand_name' => !empty($brand_name) ? $brand_name : false,
            'widgetlight' => !empty($widgetlight) ? $widgetlight : false,
            'snippets_complete' => !empty($snippets_complete) ? $snippets_complete : false,
            'snippets_active' => !empty($snippets_active) ? $snippets_active : false,
            'snippets_type' => !empty($snippets_type) ? $snippets_type : false,
            'rich_snippets_microdata' => _PS_ROOT_DIR_ . '/modules/netreviews/views/templates/hook/rich_snippets_microdata.tpl',
            'rich_snippets_json' => _PS_ROOT_DIR_ . '/modules/netreviews/views/templates/hook/rich_snippets_json.tpl',
            'detailsReviews' => !empty($detailsReviews) ? $detailsReviews : false,
            'use_star_format_image' => $use_star_format_image,
            'customized_star_color' => $customized_star_color,
        ]);

        $tpl = 'avisverifies-extraright';

        return $this->displayTemplate($tpl);
    }

    public function hookExtraRight($params)
    {
        $av_extra_option = Configuration::get('AV_EXTRA_OPTION', null, null, $this->current_shop_id);
        if ('0' == $av_extra_option) {
            return $this->hookExtraNetreviews();
        }
    }

    public function hookExtraLeft($params)
    {
        $av_extra_option = Configuration::get('AV_EXTRA_OPTION', null, null, $this->current_shop_id);
        if ('1' == $av_extra_option) {
            return $this->hookExtraNetreviews();
        }
    }

    public function hookDisplayProductButtons($params)
    {
        $av_extra_option = Configuration::get('AV_EXTRA_OPTION', null, null, $this->current_shop_id);
        if ('2' == $av_extra_option) {
            return $this->hookExtraNetreviews();
        }
    }

    /**
     * Integration of widget site on the left column.
     *
     * @return string|void
     */
    public function hookDisplayLeftColumn($params)
    {
        if ('checked' === Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id)) {
            if (null !== $this->group_name) {
                $av_scriptfixe_allowed = Configuration::get(
                    'AV_SCRIPTFIXE_ALLOWED' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
                $av_scriptfixe_position = Configuration::get(
                    'AV_SCRIPTFIXE_POSITION' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
                $av_scriptfixe = Configuration::get(
                    'AV_SCRIPTFIXE' . $this->group_name,
                    null,
                    null,
                    $this->current_shop_id
                );
            } else {
                $av_scriptfixe_allowed = 'no';
            }
        } else {
            $av_scriptfixe_allowed = Configuration::get('AV_SCRIPTFIXE_ALLOWED', null, null, $this->current_shop_id);
            $av_scriptfixe_position = Configuration::get('AV_SCRIPTFIXE_POSITION', null, null, $this->current_shop_id);
            $av_scriptfixe = Configuration::get('AV_SCRIPTFIXE', null, null, $this->current_shop_id);
        }
        if ('yes' != $av_scriptfixe_allowed || 'left' != $av_scriptfixe_position) {
            return;
        }
        if ((true != strpos(Tools::strtolower($av_scriptfixe), 'null') ||
                Tools::strlen($av_scriptfixe) > 10) && 'yes' === $av_scriptfixe_allowed) {
            return "\n\n<div id='wigetfix_avisverifies_rightcolumn'>" . Tools::stripslashes(
                html_entity_decode($av_scriptfixe)
            ) .
                '</div>';
        }
    }

    protected function displayTemplate($tpl)
    {
        // if (version_compare(_PS_VERSION_, '1.6', '<')) {
        return $this->display(__FILE__, "/views/templates/hook/$tpl.tpl");
        // } else {
        //     return  ($this->display(__FILE__, "/views/templates/hook/$tpl.tpl"));
        // }
    }

    protected function smartyAssign($smarty_array)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            global $smarty;

            return $smarty->assign($smarty_array);
        } elseif (version_compare(_PS_VERSION_, '1.5', '>=')) {
            return $this->context->smarty->assign($smarty_array);
        }
    }

    // private function initContext()
    // {
    //   if (class_exists('Context')) {
    //     $this->context = Context::getContext();
    //   } else {
    //     global $smarty, $cookie;
    //     $this->context = new StdClass();
    //     $this->context->smarty = $smarty;
    //     $this->context->cookie = $cookie;
    //   }
    // }

    protected function addFiles($file_name, $type)
    {
        if (version_compare(_PS_VERSION_, '1.4', '>=')) {
            if ('css' == $type) {
                if (version_compare(_PS_VERSION_, '1.5', '<')) {
                    return Tools::addCSS($this->_path . 'views/css/' . $file_name . '.css', 'all');
                } else {
                    return $this->context->controller->addCSS($this->_path . 'views/css/' . $file_name . '.css', 'all');
                }
            } elseif ('js' == $type) {
                if (version_compare(_PS_VERSION_, '1.5', '<')) {
                    return Tools::addJS($this->_path . 'views/js/' . $file_name . '.js', 'all');
                } else {
                    return $this->context->controller->addJS($this->_path . 'views/js/' . $file_name . '.js', 'all');
                }
            }
        }
    }

    public function hookHeader($params)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            return $this->hookDisplayHeader($params);
        }
    }

    /**
     * Return TagJS Script.
     */
    public function getTagJSScript()
    {
        $display_prod_reviews = configuration::get('AV_DISPLAYPRODREVIEWS' . $this->group_name, null, null, $this->current_shop_id);
        $av_script = configuration::get('AV_SCRIPTTAGJS' . $this->group_name, null, null, $this->current_shop_id);
        $av_script = Tools::stripslashes(html_entity_decode($av_script));
        $display_prod_reviews = ('yes' == $display_prod_reviews);
        // Please note TagJS requires following variables, ScriptTagJS, AV_DISPLAYPRODREVIEWS
        if (!empty($av_script) && $display_prod_reviews) {
            return "\n" . $av_script;
        } else {
            // Then store / storefront is active for TagJS but has either chosen no to display reviews or is missing configuration variables (Tag or Script or both)
            return;
        }
    }

    public function hookFooter($params)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            return $this->hookDisplayFooter($params);
        }
    }

    public function hookRightColumn($params)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            return $this->hookDisplayRightColumn($params);
        }
    }

    public function hookLeftColumn($params)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            return $this->hookDisplayLeftColumn($params);
        }
    }

    public function hookOrderConfirmation($params)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            return $this->hookActionValidateOrder($params);
        }
    }

    public function hookupdateOrderStatus($params)
    {
        if (version_compare(_PS_VERSION_, '1.5', '<')) {
            return $this->hookActionOrderStatusPostUpdate($params);
        }
    }

    protected function getIdConfigurationGroup($iso_lang = null)
    {
        $multisite = Configuration::get('AV_MULTISITE');

        if (1 == Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE') && !empty($multisite)) {
            $sql = 'SELECT name FROM ' . _DB_PREFIX_ . "configuration where name like 'AV_GROUP_CONF_%' And id_shop = '"
                . $this->current_shop_id . "'";
        } else {
            $sql = 'SELECT name FROM ' . _DB_PREFIX_ . "configuration where name like 'AV_GROUP_CONF_%'";
        }
        if ($results = Db::getInstance()->ExecuteS($sql)) {
            foreach ($results as $row) {
                // if (Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE') == 1 && !empty($multisite)) {
                $vconf = NetReviewsModel::avJsonDecode(Configuration::get($row['name'], null, null, $this->current_shop_id));
                /* } elseif () {

                } else                    {
                    $vconf = NetReviewsModel::avJsonDecode(Configuration::get($row['name']));
                } */
                if ($vconf && in_array($iso_lang, $vconf)) {
                    return '_' . Tools::substr($row['name'], 14);
                }
            }
        }
    }

    protected function setIdConfigurationGroup($languages = null, $idshop_conf = true, $i = 0)
    {
        if (empty($languages)) {
            return;
        }
        reset($languages);
        $id_langue_curent = key($languages);
        $lang = $languages[$id_langue_curent];

        $id_website_current = trim(Tools::getValue('avisverifies_idwebsite_' . $lang['iso_code']));
        $cle_secrete_current = trim(Tools::getValue('avisverifies_clesecrete_' . $lang['iso_code']));

        if (empty($id_website_current) || empty($cle_secrete_current)) {
            unset($languages[$id_langue_curent]);

            return $this->setIdConfigurationGroup($languages, $idshop_conf, $i);
        } else {
            if (1 == Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE') && $idshop_conf) {
                $added_condition = 'And id_shop = ' . (int) $this->current_shop_id;
            } else {
                $added_condition = '';
            }

            $sql = 'SELECT name
                FROM ' . _DB_PREFIX_ . "configuration
                WHERE value = '" . pSQL($id_website_current) . "'
                AND name like 'AV_IDWEBSITE_%' " . $added_condition;

            if ($row = Db::getInstance()->getRow($sql)) {
                if (Configuration::get(
                    'AV_CLESECRETE_' . Tools::substr($row['name'], 13),
                    null,
                    null,
                    $this->current_shop_id
                ) != $cle_secrete_current) {
                    $this->context->controller->errors[] = sprintf(
                        $this->l('PARAM ERROR: 
                        please check your multilingual configuration for 
                        the id_website "%s" at language "%s"'),
                        $id_website_current,
                        $lang['name']
                    );
                    unset($languages[$id_langue_curent]);

                    return $this->setIdConfigurationGroup($languages, $idshop_conf, $i);
                }
            }

            $group = [];
            array_push($group, $lang['iso_code']);
            unset($languages[$id_langue_curent]);

            foreach ($languages as $lang1) {
                if ($id_website_current == Tools::getValue('avisverifies_idwebsite_' . $lang1['iso_code'])
                    && $cle_secrete_current == Tools::getValue('avisverifies_clesecrete_' . $lang1['iso_code'])) {
                    array_push($group, $lang1['iso_code']);

                    $this->context->controller->errors[] = sprintf(
                        $this->l('PARAM ERROR: 
                        please check your multilingual configuration for 
                        the id_website "%s" at language "%s"'),
                        $id_website_current,
                        $lang['name']
                    );
                    // unset($languages[$id1]);
                    return $this->setIdConfigurationGroup($languages, $idshop_conf, $i);
                }
            }
            // Create PS configuration variable
            if ($idshop_conf) {
                $idshop = $this->context->shop->getContextShopID();
            } else {
                if (version_compare(_PS_VERSION_, '1.5', '<')) {
                    $idshop = null;
                } else {
                    $idshop = $this->current_shop_id;
                }
            }
            if (!Configuration::get('AV_IDWEBSITE_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_IDWEBSITE_' . $i, $id_website_current, false, null, $idshop);
            }

            if (!Configuration::get('AV_CLESECRETE_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_CLESECRETE_' . $i, $cle_secrete_current, false, null, $idshop);
            }

            if (!Configuration::get('AV_GROUP_CONF_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_GROUP_CONF_' . $i, NetReviewsModel::avJsonEncode($group), false, null, $idshop);
            }

            if (!Configuration::get('AV_PROCESSINIT_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_PROCESSINIT_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_ORDERSTATESCHOOSEN_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_ORDERSTATESCHOOSEN_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_RELOADED_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_RELOADED_' . $i, '', false, null, $idshop);
            }
            if (!Configuration::get('AV_ACTIVATE_COLLECT_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_ACTIVATE_COLLECT_' . $i, '', false, null, $idshop);
            }
            if (!Configuration::get('AV_CLIENTID_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_CLIENTID_' . $i, '', false, null, $idshop);
            }
            if (!Configuration::get('AV_SECRETAPI_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_SECRETAPI_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_DELAY_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_DELAY_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_DELAY_PRODUIT_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_DELAY_PRODUIT_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_GETPRODREVIEWS_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_GETPRODREVIEWS_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_DISPLAYPRODREVIEWS_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_DISPLAYPRODREVIEWS_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_SCRIPTFLOAT_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_SCRIPTFLOAT_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_SCRIPTFLOAT_ALLOWED_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_SCRIPTFLOAT_ALLOWED_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_SCRIPTFIXE_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_SCRIPTFIXE_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_SCRIPTFIXE_ALLOWED_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_SCRIPTFIXE_ALLOWED_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_SCRIPTFIXE_POSITION' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_SCRIPTFIXE_POSITION' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_URLCERTIFICAT_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_URLCERTIFICAT_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_FORBIDDEN_EMAIL_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_FORBIDDEN_EMAIL_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_CODE_LANG_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_CODE_LANG_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_COLLECT_CONSENT_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_COLLECT_CONSENT_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_SCRIPTTAGJS_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_SCRIPTTAGJS_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_AVERAGETAGJS_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_AVERAGETAGJS_' . $i, '', false, null, $idshop);
            }

            if (!Configuration::get('AV_REVIEWSTAGJS_' . $i, null, null, $idshop)) {
                Configuration::updateValue('AV_REVIEWSTAGJS_' . $i, '', false, null, $idshop);
            }

            ++$i;

            return $this->setIdConfigurationGroup($languages, $idshop_conf, $i);
        }
    }

    protected function sendOrders() // Function for "HOOK post status order updated"
    {
        $db = \Db::getInstance();
        // Permet de rendre optionel la demande d'avis pour les id produit contenu dans ce tableau.
        $product_exception = [// 15
        ];
        // Permet de rendre optionel la demande d'avis pour les marketplace contenu dans ce tableau.
        $global_marketplaces = [// 1 => 'priceminister'
        ];
        // Ici un tableau d'id catégories autorisées.
        $idcategories = [// 3
        ];
        // Ici un tableau d'id catégories et sous catégories a tester.
        $idcategorietotest = [];

        foreach ($idcategories as $idcat) {
            array_push($idcategorietotest, $idcat);
            $tabcat = getcategoriesrecurcive($idcat);
            foreach ($tabcat as $cat) {
                array_push($idcategorietotest, $cat);
            }
        }

        $reponse = [];
        $order_statut_list = OrderState::getOrderStates((int) Configuration::get('PS_LANG_DEFAULT'));
        $order_statut_indice = [];
        foreach ((array) $order_statut_list as $value) {
            $order_statut_indice[$value['id_order_state']] = $value['name'];
        }

        $id_shop_filter_sql = (!empty($this->current_shop_id)) ? ' AND id_shop = ' . (int) $this->current_shop_id : '';
        $query_id_shop = (!empty($this->current_shop_id)) ? ' AND o.id_shop = ' . (int) $this->current_shop_id : '';
        $allowed_products = Configuration::get('AV_GETPRODREVIEWS' . $this->group_name, null, null, $this->current_shop_id);
        $allowed_products_purchase_events = 'yes' == $allowed_products ? 'BRAND_AND_PRODUCT' : 'PURCHASE_ONLY';
        $process_choosen = Configuration::get('AV_PROCESSINIT' . $this->group_name, null, null, $this->current_shop_id);
        $order_status_choosen = Configuration::get('AV_ORDERSTATESCHOOSEN' . $this->group_name, null, null, $this->current_shop_id);
        $forbidden_mail_extensions = explode(';', Configuration::get(
            'AV_FORBIDDEN_EMAIL' . $this->group_name,
            null,
            null,
            $this->current_shop_id
        ));
        $query_iso_lang = '';
        $query_status = '';
        if ('onorderstatuschange' == $process_choosen && !empty($order_status_choosen)) {
            $order_status_choosen = str_replace(';', ',', $order_status_choosen);
            $query_status = ' AND o.current_state IN (' . pSQL($order_status_choosen) . ')';
        }
        if ('checked' == Configuration::get('AV_MULTILINGUE', null, null, $this->current_shop_id)) {
            $sql = 'SELECT value FROM ' . _DB_PREFIX_ . 'configuration WHERE name = 
            "AV_GROUP_CONF' . pSQL($this->group_name) . '"' . $id_shop_filter_sql;
            if ($row = $db->getRow($sql)) {
                $list_iso_lang_multilingue = NetReviewsModel::avJsonDecode($row['value']);
            }
            $ids_lang = '';
            foreach ($list_iso_lang_multilingue as $code_iso) {
                $o_lang = new Language();
                $id_lang = $o_lang->getIdByIso(Tools::strtolower($code_iso));
                $ids_lang .= "'" . (int) $id_lang . "',";
            }
            $ids_lang = Tools::substr($ids_lang, 0, -1);
            $query_iso_lang .= ' AND o.id_lang in (' . $ids_lang . ')';
        }

        $query = 'SELECT  o.module, oav.id_order, o.date_add as date_order, oh.date_add as 
        date_last_status,o.id_customer,o.total_paid,o.id_lang,
      o.id_shop, oh.id_order_state, o.current_state as state_order
            FROM ' . _DB_PREFIX_ . 'av_orders oav
            LEFT JOIN ' . _DB_PREFIX_ . 'orders o
            ON oav.id_order = o.id_order
            LEFT JOIN ' . _DB_PREFIX_ . 'order_history oh
            ON oh.id_order = o.id_order
            WHERE (oav.flag_get IS NULL OR oav.flag_get = 0)
            AND o.module NOT IN ("' . pSQL(implode('", "', $global_marketplaces)) . '")'
            . $query_status . $query_id_shop . $query_iso_lang . ' GROUP BY oav.id_order';

        $orders_list = $db->ExecuteS($query);
        $numRows = $db->numRows();
        $reponse['debug'][] = $query;
        $reponse['debug']['mode'] = '[' . $process_choosen . '] ' . $numRows . ' commandes récupérées';
        $orders_list_toreturn = [];

        // Clients DE et consentement pour la récupération de commande
        $ordersWithoutConsentDE = json_decode(Configuration::get(
            'AV_CONSENT_ANSWER_NO' . $this->group_name,
            null,
            null,
            $this->current_shop_id,
            false
        ), true);

        if ($orders_list) {
            foreach ($orders_list as $order) {
                // On flagge la commande à 1 si le consommateur DE n'a pas souhaité
                // que la commande soit récupérée + on nettoie la clé de config
                if (is_array($ordersWithoutConsentDE)) {
                    if (in_array((int) $order['id_order'], array_values($ordersWithoutConsentDE))) {
                        $db->Execute(
                            'UPDATE ' . _DB_PREFIX_ . 'av_orders
                            SET horodate_get = "' . time() . '", flag_get = 1
                            WHERE id_order = ' . (int) $order['id_order']
                        );
                        // }

                        $key = array_search((int) $order['id_order'], $ordersWithoutConsentDE);
                        unset($ordersWithoutConsentDE[$key]);
                        Configuration::updateValue('AV_CONSENT_ANSWER_NO' . $this->group_name, json_encode(
                            array_values($ordersWithoutConsentDE)
                        ), false, null, $this->current_shop_id);
                        continue;
                    }
                }

                // Test if customer email domain is forbidden (marketplaces case)
                $o_customer = new Customer($order['id_customer']);
                $customer_email_extension = explode('@', $o_customer->email);
                $find_occurence_forbidden_email = [];
                foreach ($forbidden_mail_extensions as $forbidden_mail) {
                    if (!empty($forbidden_mail)) {
                        if (false === strpos($customer_email_extension[1], $forbidden_mail)) {
                            $find_occurence_forbidden_email[] = 'no';
                        } else {
                            $find_occurence_forbidden_email[] = 'found';
                        }
                    }
                }

                $o_order = new Order($order['id_order']);

                if (!in_array('found', $find_occurence_forbidden_email)) {
                    $delay_product_get = Configuration::get(
                        'AV_DELAY_PRODUIT' . $this->group_name,
                        null,
                        null,
                        $order['id_shop']
                    );
                    $order_reference = (isset($o_order->reference) &&
                        !empty($o_order->reference)) ? $o_order->reference : '';
                    $delay = Configuration::get('AV_DELAY' . $this->group_name, null, null, $this->current_shop_id);
                    $delay_product = Configuration::get('AV_DELAY_PRODUIT' . $this->group_name, null, null, $this->current_shop_id) + Configuration::get('AV_DELAY' . $this->group_name, null, null, $this->current_shop_id);

                    $array_order = [
                        'purchase_date' => $order['date_order'],
                        'purchase_reference' => $order['id_order'],
                        'price' => is_numeric($order['total_paid']) ? (float) $order['total_paid'] : $order['total_paid'],
                        'consumer' => [
                            'hide_personal_data' => false,
                            'first_name' => $o_customer->firstname,
                            'last_name' => $o_customer->lastname,
                            'email' => $o_customer->email,
                        ],
                        'solicitation_parameters' => [
                            'delay' => is_numeric($delay) ? (int) $delay : 0,
                            'delay_product' => $delay_product,
                            'purchase_event_type' => $allowed_products_purchase_events,
                        ],
                        'products' => [],
                        'sales_channel' => [
                            'channel' => 'online',
                            'website_id' => Configuration::get('AV_IDWEBSITE' . $this->group_name, null, null, $this->current_shop_id),
                            'shop_id' => $order['id_shop'],
                        ],
                    ];
                    //  Add products to array
                    if (!empty($allowed_products) && 'yes' == $allowed_products) {
                        $products_in_order = $o_order->getProducts();
                        $array_products = [];
                        $i = 0;
                        $max_product = (int) Configuration::get(
                            'AV_NBOPRODUCTS' . $this->group_name,
                            null,
                            null,
                            $order['id_shop']
                        );
                        $shop_name = Configuration::get('PS_SHOP_NAME');
                        foreach ($products_in_order as $product_element) {
                            if (!in_array($product_element['product_id'], $product_exception)) {
                                if (isset($product_element['id_category_default']) &&
                                    !empty($product_element['id_category_default'])) {
                                    $product_category_create = new Category(
                                        $product_element['id_category_default'],
                                        $order['id_lang']
                                    );
                                    $product_category = $product_category_create->name;
                                } else {
                                    $product_category = '';
                                }

                                if (isset($product_element['id_manufacturer']) &&
                                    !empty($product_element['id_manufacturer'])) {
                                    $o_manufacturer = new Manufacturer($product_element['id_manufacturer']);
                                    $brand_name = $o_manufacturer->name;
                                } else {
                                    $brand_name = '';
                                }

                                $product_title_image_id = (isset($product_element['image']) &&
                                    !empty($product_element['image'])) ? $product_element['image']->id_image : '';
                                $upc = (isset($product_element['upc']) &&
                                    !empty($product_element['upc'])) ? $product_element['upc'] : '';
                                $ean13 = (isset($product_element['ean13']) &&
                                    !empty($product_element['ean13'])) ? $product_element['ean13'] : '';
                                $sku = (isset($product_element['reference']) &&
                                    !empty($product_element['reference'])) ? $product_element['reference'] : '';

                                if (isset($product_element['supplier_reference']) &&
                                    !empty($product_element['supplier_reference'])) {
                                    $mpn = $product_element['supplier_reference'];
                                } elseif (isset($product_element['product_mpn']) &&
                                    !empty($product_element['product_mpn'])) {
                                    $mpn = $product_element['product_mpn'];
                                } elseif (isset($product_element['mpn']) && !empty($product_element['mpn'])) {
                                    $mpn = $product_element['mpn'];
                                } else {
                                    $mpn = '';
                                }
                                // si déclinaisons
                                $product_name = $product_element['product_name'];
                                if (false !== strpos($product_name, '-') && false !== strpos($product_name, ':') &&
                                    Tools::strlen($product_name) > 50) {
                                    $product_name_array = explode(' - ', $product_name);
                                    $product_name = $product_name_array[0];
                                }

                                $uniquegoogleshoppinginfo = Configuration::get(
                                    'AV_PRODUCTUNIGINFO',
                                    null,
                                    null,
                                    $order['id_shop']
                                );
                                if (1 == $uniquegoogleshoppinginfo) {
                                    $product_upc = $upc;
                                    $product_ean13 = $ean13;
                                    $product_sku = $sku;
                                    $product_mpn = $mpn;
                                } else {
                                    $product_upc = (isset($product_element['product_upc']) &&
                                        !empty($product_element['product_upc'])) ?
                                        $product_element['product_upc'] : $upc;
                                    $product_ean13 = (isset($product_element['product_ean13']) &&
                                        !empty($product_element['product_ean13'])) ?
                                        $product_element['product_ean13'] : $ean13;
                                    $product_sku = (isset($product_element['product_reference']) &&
                                        !empty($product_element['product_reference'])) ?
                                        $product_element['product_reference'] : $sku;
                                    $product_mpn = (isset($product_element['product_supplier_reference']) &&
                                        !empty($product_element['product_supplier_reference'])) ?
                                        $product_element['product_supplier_reference'] : $mpn;
                                }

                                // Add Google Shopping array key product info only if data exist.
                                $product_ref_array = [
                                    'reference' => $product_element['product_id'],
                                ];
                                if (!empty($product_ean13)) {
                                    $product_ref_array = array_merge($product_ref_array, ['ean' => $product_ean13]);
                                }
                                if (!empty($product_sku)) {
                                    $product_ref_array = array_merge($product_ref_array, ['sku' => $product_sku]);
                                }
                                if (!empty($product_upc)) {
                                    $product_ref_array = array_merge($product_ref_array, ['upc' => $product_upc]);
                                }
                                if (!empty($product_mpn)) {
                                    $product_ref_array = array_merge($product_ref_array, ['mpn' => $product_mpn]);
                                }

                                // Product data format for Purchase Event
                                $product = [
                                    'product_ref' => $product_ref_array,
                                    'name' => $product_name,
                                    'brand' => (isset($brand_name) && !empty($brand_name)) ? $brand_name : $shop_name,
                                    'not_received' => false,
                                    'price' => $product_element['product_price'],
                                    'product_url' => NetReviewsModel::getUrlProduct(
                                        $product_element['product_id'],
                                        $order['id_lang']
                                    ),
                                    'image_url' => NetReviewsModel::getUrlImageProduct(
                                        $product_element['product_id'],
                                        $product_title_image_id,
                                        $order['id_lang']
                                    ),
                                    'delay' => (is_null($delay_product_get) ? 0 : is_numeric($delay_product_get)) ? (int) $delay_product_get : 0,
                                    'category' => $product_category,
                                ];
                                // limit product reviews
                                if (isset($max_product) && !empty($max_product)) {
                                    if ($max_product > 0 && $i < $max_product) {
                                        array_push($array_products, $product);
                                    }
                                } else {
                                    array_push($array_products, $product);
                                }
                                unset($product);
                            }
                            ++$i;
                        }
                        $array_order['products'] = $array_products;
                        unset($array_products);
                    }

                    if (1 == $numRows) { // Format in the right way for Purchase Events both cases.
                        $orders_list_toreturn = $array_order;
                    } else {
                        $orders_list_toreturn[] = $array_order;
                    }
                } else {
                    $reponse['message']['Emails_Interdits'][] = 'Commande n°' .
                        $order['id_order'] . ' Email:' . $o_customer->email;
                }
            }
        } // end check variable orders_list

        // Purge Table
        Db::getinstance()->Execute('DELETE FROM ' . _DB_PREFIX_ . 'av_orders WHERE 
            horodate_now < DATE_SUB(NOW(), INTERVAL 6 
            MONTH)');

        if (1 == $numRows) { // 1 order.
            $to_replyAccessToken = $this->callApiGetAccessToken(); // Get token from mulesoft to use PuchaseEvents API.
            $jsonObjectToken = json_decode($to_replyAccessToken[0]); // json_decode() is used to pass the json object to text in order to display it in html, otherwhise the json object it is not visible in html.
            if (200 == $to_replyAccessToken[1]) {
                $accessTokenApiPE = $jsonObjectToken->access_token;
                $orderJsonData = json_encode($orders_list_toreturn, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
                $to_replyPurchaseEvents = $this->callApiPostPurchaseEvents($accessTokenApiPE, $orderJsonData);
                if (200 == $to_replyPurchaseEvents['responseCode'] || 201 == $to_replyPurchaseEvents['responseCode'] || 409 == $to_replyPurchaseEvents['responseCode'] || 400 == $to_replyPurchaseEvents['responseCode']) {
                    // FLAG ORDERS TO 1
                    $jsonToArray = json_decode($orderJsonData, true);
                    $db->Execute(
                        'UPDATE ' . _DB_PREFIX_ . 'av_orders
                            SET horodate_get = "' . time() . '", flag_get = 1
                            WHERE id_order = ' . (int) $jsonToArray['purchase_reference']
                    );
                }
            }
        } elseif ($numRows > 1) { // Many orders.
            $to_replyAccessToken = $this->callApiGetAccessToken();
            $jsonObjectToken = json_decode($to_replyAccessToken[0]); // json_decode() is used to pass the json object to text in order to display it in html, otherwhise the json object it is not visible in html.

            if (200 == $to_replyAccessToken[1]) {
                $accessTokenApiPE = $jsonObjectToken->access_token;
                $orderJsonData = json_encode($orders_list_toreturn, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
                $to_replyPurchaseEvents = $this->callApiPostPurchaseEventsBulkInsert($accessTokenApiPE, $orderJsonData);
                if (200 == $to_replyPurchaseEvents['responseCode'] || 201 == $to_replyPurchaseEvents['responseCode'] || 202 == $to_replyPurchaseEvents['responseCode'] || 206 == $to_replyPurchaseEvents['responseCode'] || 409 == $to_replyPurchaseEvents['responseCode'] || 400 == $to_replyPurchaseEvents['responseCode']) { // 409 = Send the same request a second time without changing anything from the body.
                    foreach (json_decode($orderJsonData, true) as $element) {
                        // FLAG ORDERS TO 1
                        $db->Execute(
                            'UPDATE ' . _DB_PREFIX_ . 'av_orders
                            SET horodate_get = "' . time() . '", flag_get = 1
                            WHERE id_order = ' . (int) $element['purchase_reference']
                        );
                    }
                }
            }
        }
    }

    protected function callApiGetAccessToken()
    {
        $av_websiteId = Configuration::get('AV_IDWEBSITE', null, null, $this->current_shop_id);
        $av_clientid = Configuration::get('AV_CLIENTID' . $this->group_name, null, null, $this->current_shop_id);
        $av_secretapi = Configuration::get('AV_SECRETAPI' . $this->group_name, null, null, $this->current_shop_id);
        $url = 'https://auth.skeepers.io/am/oauth2/alpha/access_token';

        try {
            $curl = curl_init($url);
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, 'grant_type=client_credentials&scope=openid');
            curl_setopt($curl, CURLOPT_USERPWD, $av_clientid . ':' . $av_secretapi);
            curl_setopt($curl, CURLOPT_HTTPHEADER, [
                'Content-Type: application/x-www-form-urlencoded',
                'Host: auth.skeepers.io',
            ]);
            curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 0);
            curl_setopt($curl, CURLOPT_TIMEOUT, 60); // timeout in seconds
            $response = curl_exec($curl);
            $responseCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $curl_errno = curl_errno($curl);
            $curl_error = curl_error($curl);
            curl_close($curl);

            if (false === $response) {
                $this->addLog('CURL callApiGetAccessToken - idWebsite ' . $av_websiteId . ' - idShop ' . $this->current_shop_id . ' - responseCode (' . $responseCode . ') ERROR(' . $curl_errno . '): ' . print_r($curl_error, true), 'CURL Error');
                throw new Exception($responseCode);
            }

            if (200 == $responseCode) {
                return [$response, $responseCode];
            }
            $this->addLog('CURL callApiGetAccessToken - idWebsite ' . $av_websiteId . ' - idShop ' . $this->current_shop_id . ' - responseCode (' . $responseCode . ') ERROR(' . $curl_errno . '): ' . print_r($curl_error, true), 'CURL Error');

            return [$response, $responseCode];
        } catch (Exception $e) {
            $this->addLog('Error in function callApiGetAccessToken - idWebsite ' . $av_websiteId . ' - idShop ' . $this->current_shop_id . ' -  responseCode (' . $e->getMessage() . ')', 'CURL Error');
        }
    }

    protected function callApiPostPurchaseEvents($token, $order)
    {
        $websiteId = Configuration::get('AV_IDWEBSITE', null, null, $this->current_shop_id);
        $urlPEDev = 'https://api.skeepers.io/purchase-event/websites/' . $websiteId . '/purchase_events';
        try {
            $curl = curl_init($urlPEDev);
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $order);
            curl_setopt($curl, CURLOPT_HTTPHEADER, [
                'Content-Type: application/json',
                'Authorization: Bearer ' . $token,
                'Host: api.skeepers.io',
                'x-api-key: hIqW6zqjUq9qorWSlVgX6HSZM4FWCPy33nLjiD0b',
                'x-custom-loading-mode: API_PRESTASHOP',
            ]);
            curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 0);
            curl_setopt($curl, CURLOPT_TIMEOUT, 3600); // timeout in seconds = 1h
            $response = curl_exec($curl);
            $responseCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $curl_errno = curl_errno($curl);
            $curl_error = curl_error($curl);
            curl_close($curl);

            $orderJsonData = json_decode($order, true);
            $orderReference = $orderJsonData['purchase_reference'];

            if (false === $response) {
                $this->addLog('CURL callApiPostPurchaseEvents - idWebsite ' . $websiteId . ' - idShop ' . $this->current_shop_id . ' - responseCode (' . $responseCode . ') for order reference (' . $orderReference . ') ERROR(' . $curl_errno . ')', 'CURL Error');
                throw new Exception($responseCode);
            }

            if (200 == $responseCode || 201 == $responseCode || 202 == $responseCode) {
                return ['response' => $response, 'responseCode' => $responseCode];
            }

            $returnCurlToString = htmlspecialchars_decode(print_r($response, true));
            $returnCurl = str_replace(["'", '"', '&quot;'], '', $returnCurlToString);
            $this->addLog('CURL callApiPostPurchaseEvents - idWebsite ' . $websiteId . ' - idShop ' . $this->current_shop_id . ' - responseCode (' . $responseCode . ') for order reference (' . $orderReference . ') ERROR(' . $curl_errno . ')', 'CURL Error');

            return ['response' => $response, 'responseCode' => $responseCode];
        } catch (Exception $e) {
            $this->addLog('Error in function callApiPostPurchaseEvents - idWebsite ' . $websiteId . ' - idShop ' . $this->current_shop_id . ' - responseCode (' . $e->getMessage() . ')', 'CURL Error');
        }
    }

    protected function callApiPostPurchaseEventsBulkInsert($token, $orders)
    {
        $websiteId = Configuration::get('AV_IDWEBSITE', null, null, $this->current_shop_id);
        $urlPEDev = 'https://api.skeepers.io/purchase-event/websites/' . $websiteId . '/purchase_events/bulk_insert';

        try {
            $curl = curl_init($urlPEDev);
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $orders);
            curl_setopt($curl, CURLOPT_HTTPHEADER, [
                'Content-Type: application/json',
                'Authorization: Bearer ' . $token,
                'Host: api.skeepers.io',
                'x-api-key: hIqW6zqjUq9qorWSlVgX6HSZM4FWCPy33nLjiD0b',
                'x-custom-loading-mode: API_PRESTASHOP',
            ]);
            curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 0);
            curl_setopt($curl, CURLOPT_TIMEOUT, 3600); // timeout in seconds = 1h
            $response = curl_exec($curl);
            $responseCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $curl_errno = curl_errno($curl);
            $curl_error = curl_error($curl);
            curl_close($curl);

            $ordersJsonData = json_decode($orders, true);
            foreach ($ordersJsonData as $element) {
                $ordersReference[] = $element['purchase_reference'];
            }

            if (false === $response) {
                $this->addLog('CURL callApiPostPurchaseEventsBulkInsert - idWebsite ' . $websiteId . ' - idShop ' . $this->current_shop_id . ' - responseCode (' . $responseCode . ') for order references (' . print_r($ordersReference, true) . ') ERROR(' . $curl_errno . ')', 'CURL Error');
                throw new Exception($responseCode);
            }

            if (200 == $responseCode || 201 == $responseCode || 202 == $responseCode) {
                return ['response' => $response, 'responseCode' => $responseCode];
            }

            $returnCurlToString = htmlspecialchars_decode(print_r($response, true));
            $returnCurl = str_replace(["'", '"', '&quot;'], '', $returnCurlToString);
            $this->addLog('CURL callApiPostPurchaseEventsBulkInsert - idWebsite ' . $websiteId . ' - idShop ' . $this->current_shop_id . ' - responseCode (' . $responseCode . ') for order references (' . print_r($ordersReference, true) . ') ERROR(' . $curl_errno . ')', 'CURL Error');

            return ['response' => $response, 'responseCode' => $responseCode];
        } catch (Exception $e) {
            $this->addLog('Error in function callApiPostPurchaseEventsBulkInsert - idWebsite ' . $websiteId . ' - idShop ' . $this->current_shop_id . ' - responseCode (' . $e->getMessage() . ')', 'CURL Error');
        }
    }
}
